# Django
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required

# AA TaxSystem
from taxsystem.models.alliance import (
    AllianceFilter,
    AllianceFilterSet,
    AlliancePaymentAccount,
    AlliancePayments,
)
from taxsystem.models.corporation import (
    CorporationFilter,
    CorporationFilterSet,
    CorporationPaymentAccount,
    CorporationPayments,
    Members,
)
from taxsystem.models.helpers.textchoices import AccountStatus, PaymentRequestStatus


@permissions_required(
    [
        "taxsystem.manage_own_corp",
        "taxsystem.manage_corps",
        "taxsystem.manage_own_alliance",
        "taxsystem.manage_alliances",
    ]
)
def get_taxsystem_manage_action_icons(
    request: WSGIRequest,  # pylint: disable=unused-argument
    account: CorporationPaymentAccount | AlliancePaymentAccount,
    checkbox: bool = False,
) -> str | HttpResponse:
    """
    Generate HTML Action Icons for the Tax System Management view.

    This function creates a set of action icons for managing tax system accounts.
    The Buttons include Edit, Delete, and View Transactions, each represented by an icon depending on User's permissions.

    Args:
        request (WSGIRequest): The HTTP request object containing user information.
        account (CorporationPaymentAccount | AlliancePaymentAccount): The tax account object.
        checkbox (bool): Whether to include a checkbox for bulk actions.
    Returns:
        SafeString: HTML string containing the action icons.
    """
    taxsystem_request_icons = "<div class='d-flex justify-content-end'>"
    taxsystem_request_icons += get_tax_account_add_button(account=account)
    taxsystem_request_icons += get_tax_account_switch_button(account=account)
    taxsystem_request_icons += get_tax_account_info_button(account=account)
    if checkbox:
        taxsystem_request_icons += f'<input type="checkbox" class="tax-row-select form-check-input me-2" data-account-pk="{account.pk}" />'
    taxsystem_request_icons += "</div>"
    return taxsystem_request_icons


@permissions_required(
    [
        "taxsystem.manage_own_corp",
        "taxsystem.manage_corps",
        "taxsystem.manage_own_alliance",
        "taxsystem.manage_alliances",
    ]
)
def get_taxsystem_manage_payments_action_icons(
    request: WSGIRequest,  # pylint: disable=unused-argument
    payment: CorporationPayments | AlliancePayments,
    checkbox: bool = False,
) -> str | HttpResponse:
    """
    Generate HTML Action Icons for the Tax System Management view Payments Modal.

    This function creates a set of action icons for managing tax system payments.
    The Buttons include Approve, Reject, Undo, Delete and View Details, each represented by an icon depending on User's permissions.
    optionally includes a checkbox for bulk actions.

    Args:
        request (WSGIRequest): The HTTP request object containing user information.
        payment (CorporationPayments | AlliancePayments): The payment object.
        checkbox (bool): Whether to include a checkbox for bulk actions.
    Returns:
        SafeString: HTML string containing the action icons.
    """
    taxsystem_request_icons = "<div class='d-flex justify-content-end'>"

    # Only show approve/reject buttons for pending or needs approval payments
    if payment.request_status in [
        PaymentRequestStatus.PENDING,
        PaymentRequestStatus.NEEDS_APPROVAL,
    ]:
        taxsystem_request_icons += get_payments_approve_button(payment=payment)
        taxsystem_request_icons += get_payments_reject_button(payment=payment)
    # Only show undo button for approved/rejected payments
    if payment.request_status in [
        PaymentRequestStatus.APPROVED,
        PaymentRequestStatus.REJECTED,
    ]:
        taxsystem_request_icons += get_payments_undo_button(payment=payment)
    # Only show delete button for custom payments (not linked to an Wallet Entry)
    if payment.journal is None:
        taxsystem_request_icons += get_payments_delete_button(payment=payment)
    # Always show info button
    taxsystem_request_icons += get_payments_info_button(payment=payment)
    if checkbox:
        taxsystem_request_icons += f'<input type="checkbox" class="tax-row-select form-check-input me-2" data-payment-pk="{payment.pk}" />'
    taxsystem_request_icons += "</div>"
    return taxsystem_request_icons


def get_taxsystem_payments_action_icons(
    request: WSGIRequest,
    payment: CorporationPayments | AlliancePayments,
    checkbox: bool = False,
) -> str | HttpResponse:
    """
    Generate HTML Action Icons for the Tax System Payments view.

    This function creates a set of action icons for viewing tax system payments.
    The Buttons include View Details, each represented by an icon.
    With additional buttons for users with manage permissions.
    optionally includes a checkbox for bulk actions.

    Args:
        request (WSGIRequest): The HTTP request object containing user information.
        payment (CorporationPayments | AlliancePayments): The payment object.
        checkbox (bool): Whether to include a checkbox for bulk actions.
    Returns:
        SafeString: HTML string containing the action icons.
    """
    manage_permission = [
        "taxsystem.manage_own_corp",
        "taxsystem.manage_corps",
        "taxsystem.manage_own_alliance",
        "taxsystem.manage_alliances",
    ]

    taxsystem_request_icons = "<div class='d-flex justify-content-end'>"
    taxsystem_request_icons += get_payments_info_button(payment=payment)
    if request.user.get_user_permissions().intersection(set(manage_permission)):
        # Only show approve/reject buttons for pending or needs approval payments
        if payment.request_status in [
            PaymentRequestStatus.PENDING,
            PaymentRequestStatus.NEEDS_APPROVAL,
        ]:
            taxsystem_request_icons += get_payments_approve_button(payment=payment)
            taxsystem_request_icons += get_payments_reject_button(payment=payment)
        # Only show undo button for approved/rejected payments
        if payment.request_status in [
            PaymentRequestStatus.APPROVED,
            PaymentRequestStatus.REJECTED,
        ]:
            taxsystem_request_icons += get_payments_undo_button(payment=payment)
        # Only show delete button for custom payments (not linked to an Wallet Entry)
        if payment.journal is None:
            taxsystem_request_icons += get_payments_delete_button(payment=payment)
        if checkbox:
            taxsystem_request_icons += f'<input type="checkbox" class="tax-row-select form-check-input me-2" data-payment-pk="{payment.pk}" />'
    taxsystem_request_icons += "</div>"
    return taxsystem_request_icons


@permissions_required(
    [
        "taxsystem.manage_own_corp",
        "taxsystem.manage_corps",
        "taxsystem.manage_own_alliance",
        "taxsystem.manage_alliances",
    ]
)
def get_filter_set_action_icons(
    request: WSGIRequest,  # pylint: disable=unused-argument
    filter_set: CorporationFilterSet | AllianceFilterSet,
) -> str | HttpResponse:
    """
    Generate HTML Action Icons for Manage Filter Set View.

    This function creates a set of action icons for managing Tax System Filter Sets.
    The Buttons include Edit, Delete, and View Filter Sets, each represented by an icon depending on User's permissions.

    Args:
        request (WSGIRequest): The HTTP request object containing user information.
        filter_set (CorporationFilterSet | AllianceFilterSet): The filter set object.
    Returns:
        String: HTML string containing the action icons.
    """
    taxsystem_request_icons = "<div class='d-flex justify-content-end'>"
    taxsystem_request_icons += get_filter_set_info_button(filter_set=filter_set)
    taxsystem_request_icons += get_filter_set_switch_button(filter_set=filter_set)
    taxsystem_request_icons += get_filter_set_delete_button(filter_set=filter_set)
    taxsystem_request_icons += "</div>"
    return taxsystem_request_icons


def get_filter_set_info_button(
    filter_set: CorporationFilterSet | AllianceFilterSet,
) -> str:
    """
    Generate a Filter Set Info button for the Manage Filter View.

    This function creates a HTML info button for viewing a given Filter Set object.
    When clicked, it triggers a modal to display detailed information about the Filter Set.

    Args:
        filter_set (CorporationFilterSet | AllianceFilterSet): The filter set object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the Info Request
    button_request_info_url = reverse(
        "taxsystem:api:get_filters",
        kwargs={
            "filterset_pk": filter_set.pk,
            "owner_id": filter_set.owner.eve_id,
        },
    )

    # Define the icon and tooltip for the info button
    icon = '<i class="fa-solid fa-info"></i>'
    title = _("View Filter Set")
    color = "primary"

    # Create the HTML for the info icon button
    filter_set_info_button = (
        f'<button data-action="{button_request_info_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-view-filter-set" '
        f'title="{title}">{icon}</button>'
    )
    return filter_set_info_button


def get_filter_set_switch_button(
    filter_set: CorporationFilterSet | AllianceFilterSet,
) -> str:
    """
    Generate a Filter Set Switch for the Manage Filter View.

    This function creates a HTML Switch button for switching Filter Sets Status depending on it's current state.
    When clicked, it toggles the active state of the Filter Set.

    Args:
        filter_set (CorporationFilterSet | AllianceFilterSet): The filter set object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the Filter Set Switch Request
    button_request_switch_url = reverse(
        "taxsystem:api:switch_filter_set",
        kwargs={
            "filterset_pk": filter_set.pk,
            "owner_id": filter_set.owner.eve_id,
        },
    )

    # Define the icon, color and tooltip for the switch button
    color = "warning"
    if filter_set.enabled:
        icon = '<i class="fa-solid fa-eye-low-vision"></i>'
        title = _("Deactivate Filter Set")
    else:
        icon = '<i class="fa-solid fa-eye"></i>'
        title = _("Activate Filter Set")

    # Create the HTML for the switch icon button
    filter_set_switch_button = (
        '<button id="taxsystem-switch-filter-set-button" '
        f'data-action="{button_request_switch_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-tooltip="aa-taxsystem" '
        f'title="{title}">{icon}</button>'
    )
    return filter_set_switch_button


def get_filter_set_delete_button(
    filter_set: CorporationFilterSet | AllianceFilterSet,
) -> str:
    """
    Generate a Filter Set Delete button for the Manage Filter View.

    This function creates a HTML delete button for deleting a given Filter Set object.
    When clicked, it triggers a modal to confirm the deletion of the Filter Set.

    Args:
        filter_set (CorporationFilterSet | AllianceFilterSet): The filter set object to be deleted.
    Returns:
        String: HTML string containing the delete button.
    """

    # Generate the URL for the delete Request
    button_request_delete_url = reverse(
        viewname="taxsystem:api:delete_filter_set",
        kwargs={
            "filterset_pk": filter_set.pk,
            "owner_id": filter_set.owner.eve_id,
        },
    )

    # Define the icon and tooltip for the delete button
    icon = '<i class="fa-solid fa-trash"></i>'
    title = _("Delete Filter Set")
    color = "danger"

    # Create the HTML for the delete icon button
    filter_set_delete_button = (
        f'<button data-action="{button_request_delete_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-accept-delete-filter-set" '
        f'title="{title}">{icon}</button>'
    )
    return filter_set_delete_button


def get_filter_set_active_icon(
    filter_set: CorporationFilterSet | AllianceFilterSet,
) -> str:
    """
    Generate a Filter Set Active icon for the Filter Set View.

    This function creates a HTML active icon button for showing active status of a given Filter Set object

    Args:
        filter_set (CorporationFilterSet | AllianceFilterSet): The filter set object to be viewed.
    Returns:
        String: HTML string containing the active icon button.
    """
    # Define the icon, color and tooltip for the active icon
    if filter_set.enabled:
        icon = '<i class="fa-solid fa-check"></i>'
        title = _("Active")
        color = "success"
    else:
        icon = '<i class="fa-solid fa-xmark"></i>'
        title = _("Inactive")
        color = "danger"
    # Create the HTML for the active icon button
    filter_set_active_button = (
        f"<button "
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-tooltip="aa-taxsystem" '
        f'title="{title}">{icon}</button>'
    )
    return filter_set_active_button


def get_filter_delete_button(filter_obj: CorporationFilter | AllianceFilter) -> str:
    """
    Generate a Filter Delete button for the Manage Filter View.

    This function creates a HTML delete button for deleting a given Filter object.
    When clicked, it triggers a modal to confirm the deletion of the Filter, optionally loading previous modal.

    Args:
        filter_obj (CorporationFilter | AllianceFilter): The filter object to be deleted.
    Returns:
        String: HTML string containing the delete button.
    """

    # Generate the URL for the delete Request
    button_request_delete_url = reverse(
        viewname="taxsystem:api:delete_filter",
        kwargs={
            "filter_pk": filter_obj.pk,
            "owner_id": filter_obj.filter_set.owner.eve_id,
        },
    )
    # Generate the URL for reloading the Modal Data
    modal_request_url = reverse(
        "taxsystem:api:get_filters",
        kwargs={
            "filterset_pk": filter_obj.filter_set.pk,
            "owner_id": filter_obj.filter_set.owner.eve_id,
        },
    )
    # Define the icon and tooltip for the delete button
    icon = '<i class="fa-solid fa-trash"></i>'
    title = _("Delete Filter")
    color = "danger"

    # Create the HTML for the delete icon button
    filter_delete_button = (
        f'<button data-action="{button_request_delete_url}" '
        f'data-previous-modal="{modal_request_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-accept-delete-filter" '
        f'title="{title}">{icon}</button>'
    )
    return filter_delete_button


def get_tax_account_switch_button(
    account: CorporationPaymentAccount | AlliancePaymentAccount,
) -> str:
    """
    Generate a Tax Account Switch button for the Tax System Management view.

    This function creates a HTML Switch button for switching Tax Account Status depending on it's current state.
    When clicked, it toggles the active state of the Tax Account.

    Args:
        account (CorporationPaymentAccount | AlliancePaymentAccount): The tax account object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the switch Request
    button_request_switch_url = reverse(
        "taxsystem:api:switch_tax_account",
        kwargs={
            "account_pk": account.pk,
            "owner_id": account.owner.eve_id,
        },
    )

    # Define the icon, color and tooltip for the switch button
    color = "warning"
    if account.status == AccountStatus.ACTIVE:
        icon = '<i class="fa-solid fa-eye-low-vision"></i>'
        title = _("Deactivate Account")
    else:
        icon = '<i class="fa-solid fa-eye"></i>'
        title = _("Activate Account")

    # Create the HTML for the switch icon button
    tax_account_switch_button = (
        "<button "
        f'data-action="{button_request_switch_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-accept-switch-tax-account" '
        f'title="{title}">{icon}</button>'
    )
    return tax_account_switch_button


def get_tax_account_add_button(
    account: CorporationPaymentAccount | AlliancePaymentAccount,
) -> str:
    """
    Generate a Add Payment button for the Tax System Management view.

    This function creates a HTML Add Payment button for adding payments to a tax account.
    When clicked, it triggers a modal to confirm the addition of a payment.

    Args:
        account (CorporationPaymentAccount | AlliancePaymentAccount): The tax account object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the add payment Request
    button_request_switch_url = reverse(
        "taxsystem:api:add_payment",
        kwargs={
            "account_pk": account.pk,
            "owner_id": account.owner.eve_id,
        },
    )

    # Define the icon, color and tooltip for the add payment button
    color = "success"
    icon = '<i class="fa-solid fa-dollar-sign"></i>'
    title = _("Add Payment")

    # Create the HTML for the add payment icon button
    tax_account_add_button = (
        "<button "
        f'data-action="{button_request_switch_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-accept-add-payment" '
        f'title="{title}">{icon}</button>'
    )
    return tax_account_add_button


def get_tax_account_info_button(
    account: CorporationPaymentAccount | AlliancePaymentAccount,
) -> str:
    """
    Generate a Tax Account Info button for the Tax System Management view.

    This function creates a HTML info button for viewing a Tax Account object.
    When clicked, it triggers a modal to display detailed information about the Tax Account with managing tools.

    Args:
        account (CorporationPaymentAccount | AlliancePaymentAccount): The tax account object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the info Request
    button_request_info_url = reverse(
        "taxsystem:api:get_member_payments",
        kwargs={
            "owner_id": account.owner.eve_id,
            "character_id": account.user.profile.main_character.character_id,
        },
    )

    # Define the icon and tooltip for the info button
    icon = '<i class="fa-solid fa-info"></i>'
    title = _("View Tax Account")
    color = "primary"

    # Create the HTML for the info icon button
    tax_account_info_button = (
        f'<button data-action="{button_request_info_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-view-tax-account" '
        f'title="{title}">{icon}</button>'
    )
    return tax_account_info_button


def get_payments_approve_button(payment: CorporationPayments | AlliancePayments) -> str:
    """
    Generate a Approve Payment button for the Payments Modal view.

    This function creates a HTML Approve Payment button for approving payments to the according tax account.
    When clicked, it triggers a modal to confirm the approval the according payment.

    Args:
        payment (CorporationPayments | AlliancePayments): The payment object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the approve payment Request
    button_request_approve_url = reverse(
        "taxsystem:api:approve_payment",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "payment_pk": payment.pk,
        },
    )
    # Generate the URL for reloading the Modal Data
    modal_request_url = reverse(
        "taxsystem:api:get_member_payments",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "character_id": payment.character_id,
        },
    )

    # Define the icon, color and tooltip for the approve button
    color = "success"
    icon = '<i class="fa-solid fa-check"></i>'
    title = _("Approve Payment")

    # Create the HTML for the add payment icon button
    payments_approve_button = (
        "<button "
        f'data-action="{button_request_approve_url}" '
        f'data-previous-modal="{modal_request_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-accept-approve-payment" '
        f'title="{title}">{icon}</button>'
    )
    return payments_approve_button


def get_payments_undo_button(payment: CorporationPayments | AlliancePayments) -> str:
    """
    Generate a Undo Payment button for the Payments Modal view.

    This function creates a HTML Undo Payment button for undoing payments to the according tax account.
    When clicked, it triggers a modal to confirm the undo the according payment.

    Args:
        payment (CorporationPayments | AlliancePayments): The payment object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the undo payment Request
    button_request_undo_url = reverse(
        "taxsystem:api:undo_payment",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "payment_pk": payment.pk,
        },
    )
    # Generate the URL for reloading the Modal Data
    modal_request_url = reverse(
        "taxsystem:api:get_member_payments",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "character_id": payment.character_id,
        },
    )

    # Define the icon, color and tooltip for the undo button
    color = "warning"
    icon = '<i class="fa-solid fa-undo"></i>'
    title = _("Undo Payment")

    # Create the HTML for the undo payment icon button
    payments_undo_button = (
        "<button "
        f'data-action="{button_request_undo_url}" '
        f'data-previous-modal="{modal_request_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-accept-undo-payment" '
        f'title="{title}">{icon}</button>'
    )
    return payments_undo_button


def get_payments_info_button(payment: CorporationPayments | AlliancePayments) -> str:
    """
    Generate a Information Payment button for the Payments Modal view.

    This function creates a HTML Information Payment button for viewing information about the according payment.
    When clicked, it triggers a modal to display detailed payment information.

    Args:
        payment (CorporationPayments | AlliancePayments): The payment object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the info payment Request
    button_request_info_url = reverse(
        "taxsystem:api:get_payment_details",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "payment_pk": payment.pk,
        },
    )
    # Generate the URL for reloading the Modal Data
    modal_request_url = reverse(
        "taxsystem:api:get_member_payments",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "character_id": payment.character_id,
        },
    )

    # Define the icon, color and tooltip for the info button
    color = "primary"
    icon = '<i class="fa-solid fa-info"></i>'
    title = _("Show Details")

    # Create the HTML for the info icon button
    payments_info_button = (
        "<button "
        f'data-action="{button_request_info_url}" '
        f'data-previous-modal="{modal_request_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-view-payment-details" '
        f'title="{title}">{icon}</button>'
    )
    return payments_info_button


def get_payments_delete_button(payment: CorporationPayments | AlliancePayments) -> str:
    """
    Generate a Delete Payment button for the Payments Modal view.

    This function creates a HTML Delete Payment button for deleting payments to the according tax account.
    When clicked, it triggers a modal to confirm the deletion of the according payment.

    Args:
        payment (CorporationPayments | AlliancePayments): The payment object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the delete payment Request
    button_request_delete_url = reverse(
        "taxsystem:api:delete_payment",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "payment_pk": payment.pk,
        },
    )
    # Generate the URL for reloading the Modal Data
    modal_request_url = reverse(
        "taxsystem:api:get_member_payments",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "character_id": payment.character_id,
        },
    )

    # Define the icon, color and tooltip for the delete button
    color = "danger"
    icon = '<i class="fa-solid fa-trash"></i>'
    title = _("Delete Payment")

    # Create the HTML for the delete payment icon button
    payments_delete_button = (
        "<button "
        f'data-action="{button_request_delete_url}" '
        f'data-previous-modal="{modal_request_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-accept-delete-payment" '
        f'title="{title}">{icon}</button>'
    )
    return payments_delete_button


def get_payments_reject_button(payment: CorporationPayments | AlliancePayments) -> str:
    """
    Generate a Reject Payment button for the Payments Modal view.

    This function creates a HTML Reject Payment button for rejecting payments to the according tax account.
    When clicked, it triggers a modal to confirm the rejection of the according payment.

    Args:
        payment (CorporationPayments | AlliancePayments): The payment object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the add payment Request
    button_request_reject_url = reverse(
        "taxsystem:api:reject_payment",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "payment_pk": payment.pk,
        },
    )
    # Generate the URL for reloading the Modal Data
    modal_request_url = reverse(
        "taxsystem:api:get_member_payments",
        kwargs={
            "owner_id": payment.account.owner.eve_id,
            "character_id": payment.character_id,
        },
    )

    # Define the icon, color and tooltip for the reject button
    color = "danger"
    icon = '<i class="fa-solid fa-xmark"></i>'
    title = _("Reject Payment")

    # Create the HTML for the reject payment icon button
    payments_reject_button = (
        "<button "
        f'data-action="{button_request_reject_url}" '
        f'data-previous-modal="{modal_request_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-accept-reject-payment" '
        f'title="{title}">{icon}</button>'
    )
    return payments_reject_button


def get_members_delete_button(member: Members) -> str:
    """
    Generate a Delete Member button for the Tax System Manage view.

    This function creates a HTML Delete Member button for deleting members in the Tax System Manage view.
    When clicked, it triggers a modal to confirm the deletion of the according member.

    Args:
        member (Members): The member object to be viewed.
    Returns:
        String: HTML string containing the info button.
    """

    # Generate the URL for the info member Request
    button_request_delete_url = reverse(
        "taxsystem:api:delete_member",
        kwargs={
            "owner_id": member.owner.eve_id,
            "member_pk": member.pk,
        },
    )

    # Define the icon, color and tooltip for the delete button
    color = "danger"
    icon = '<i class="fa-solid fa-trash"></i>'
    title = _("Delete Member")

    # Create the HTML for the delete member icon button
    members_delete_button = (
        "<button "
        f'data-action="{button_request_delete_url}" '
        f'class="btn btn-{color} btn-sm btn-square me-2" '
        'data-bs-toggle="modal" '
        'data-bs-tooltip="aa-taxsystem" '
        'data-bs-target="#taxsystem-accept-delete-member" '
        f'title="{title}">{icon}</button>'
    )
    return members_delete_button



# Django
from django.test import override_settings

# AA TaxSystem
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.utils import (
    create_owner_from_user,
)

MODULE_PATH = "taxsystem.managers.owner_manager"


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestOwnerManager(TaxSystemTestCase):
    """Test Corporation Managers."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.audit = create_owner_from_user(cls.user)
        cls.alliance_audit = create_owner_from_user(cls.user, tax_type="alliance")

    def test_visible_to(self):
        """
        Test OwnerManager.visible_to() method.
        This test verifies that the visibility of owners is correctly managed based on user permissions.

        Results:
            # Corporation visible_to tests
            1. The owner associated with user 1 is visible to user 1.
            2. The owner associated with user 2 is visible to user 2.
            3. Both owners are visible to a superuser.
            4. Both owners are visible to a user with 'taxsystem.manage_corps' permission.
            # Alliance visible_to tests
            1. The alliance owner associated with user 1 is visible to user 1.
            2. The alliance owner associated with user 2 is visible to user 2.
            3. Both alliance owners are visible to a superuser.
            4. Both alliance owners are visible to a user with 'taxsystem.manage_alliances' permission.
        """
        # Setup additional user and owner
        audit_2 = create_owner_from_user(self.user2)
        alliance_audit_2 = create_owner_from_user(self.user2, tax_type="alliance")

        # Test that audit is visible to its user
        visible_audits_user_1 = type(self.audit).objects.visible_to(self.user)
        self.assertIn(self.audit, visible_audits_user_1)
        self.assertNotIn(audit_2, visible_audits_user_1)

        # Test that audit_2 is visible to its user
        visible_audits_user_2 = type(audit_2).objects.visible_to(self.user2)
        self.assertIn(audit_2, visible_audits_user_2)
        self.assertNotIn(self.audit, visible_audits_user_2)

        # Test that both audits are visible to a superuser
        self.assertIn(self.audit, type(self.audit).objects.visible_to(self.superuser))
        self.assertIn(audit_2, type(audit_2).objects.visible_to(self.superuser))

        # Test that both audits are visible to permission 'taxsystem.manage_corps'
        self.assertIn(self.audit, type(self.audit).objects.visible_to(self.manage_user))
        self.assertIn(audit_2, type(audit_2).objects.visible_to(self.manage_user))

        # Alliance visible_to tests
        # Test that alliance_audit is visible to its user
        visible_alliance_audits_user_1 = type(self.alliance_audit).objects.visible_to(
            self.user
        )
        self.assertIn(self.alliance_audit, visible_alliance_audits_user_1)
        self.assertNotIn(alliance_audit_2, visible_alliance_audits_user_1)

        # Test that alliance_audit_2 is visible to its user
        visible_alliance_audits_user_2 = type(alliance_audit_2).objects.visible_to(
            self.user2
        )
        self.assertIn(alliance_audit_2, visible_alliance_audits_user_2)
        self.assertNotIn(self.alliance_audit, visible_alliance_audits_user_2)

        # Test that both alliance audits are visible to a superuser
        self.assertIn(
            self.alliance_audit,
            type(self.alliance_audit).objects.visible_to(self.superuser),
        )
        self.assertIn(
            alliance_audit_2, type(alliance_audit_2).objects.visible_to(self.superuser)
        )

        # Test that both alliance audits are visible to permission 'taxsystem.manage_alliances'
        self.assertIn(
            self.alliance_audit,
            type(self.alliance_audit).objects.visible_to(self.manage_user),
        )
        self.assertIn(
            alliance_audit_2,
            type(alliance_audit_2).objects.visible_to(self.manage_user),
        )

    def test_manage_to(self):
        """
        Test OwnerManager.manage_to() method.
        This test verifies that the management access of owners is correctly managed based on user permissions.

        Results:
            # Corporation manage_to tests
            1. The owner associated with user 1 can not be managed by user 1.
            2. The owner associated with user 1 can not be managed by other users.
            3. Both owners are manageable to a superuser.
            4. Own owner is manageable to a user with 'taxsystem.manage_own_corp' permission.
            5. Both owners are manageable to a user with 'taxsystem.manage_corps' permission.
            # Alliance manage_to tests
            1. The alliance owner associated with user 1 can not be managed by user 1.
            2. The alliance owner associated with user 1 can not be managed by other users.
            3. Both alliance owners are manageable to a superuser.
            4. Own alliance owner is manageable to a user with 'taxsystem.manage_own_alliance' permission.
            5. Both alliance owners are manageable to a user with 'taxsystem.manage_alliances' permission.
        """
        # Setup additional user and owner
        audit_2 = create_owner_from_user(self.user2)
        alliance_audit_2 = create_owner_from_user(self.user2, tax_type="alliance")

        # Test that user can not be managed by its user
        manageable_audits_user_1 = type(self.audit).objects.manage_to(self.user)
        self.assertNotIn(self.audit, manageable_audits_user_1)

        # Test that audit can not be managed by other user
        manageable_audits_user_2 = type(audit_2).objects.manage_to(self.user2)
        self.assertNotIn(audit_2, manageable_audits_user_2)
        self.assertNotIn(self.audit, manageable_audits_user_2)

        # Test that both audits are manageable to a superuser
        self.assertIn(self.audit, type(self.audit).objects.manage_to(self.superuser))
        self.assertIn(audit_2, type(audit_2).objects.manage_to(self.superuser))

        # Test that own corporation audit are manageable to permission 'taxsystem.manage_own_corp'
        self.assertIn(
            self.audit, type(self.audit).objects.manage_to(self.manage_own_user)
        )
        self.assertNotIn(audit_2, type(audit_2).objects.manage_to(self.manage_own_user))

        # Test that all corporation audits are manageable to permission 'taxsystem.manage_corps'
        self.assertIn(audit_2, type(audit_2).objects.manage_to(self.manage_user))
        self.assertIn(self.audit, type(self.audit).objects.manage_to(self.manage_user))

        # Alliance manageable_to tests
        # Test that alliance audit can not be managed by its user
        manageable_audits_user_1 = type(self.alliance_audit).objects.manage_to(
            self.user
        )
        self.assertNotIn(self.alliance_audit, manageable_audits_user_1)

        # Test that alliance audit can not be managed by other user
        manageable_audits_user_2 = type(alliance_audit_2).objects.manage_to(self.user2)
        self.assertNotIn(alliance_audit_2, manageable_audits_user_2)
        self.assertNotIn(self.alliance_audit, manageable_audits_user_2)

        # Test that both alliance audits are manageable to a superuser
        self.assertIn(
            self.alliance_audit,
            type(self.alliance_audit).objects.manage_to(self.superuser),
        )
        self.assertIn(
            alliance_audit_2, type(alliance_audit_2).objects.manage_to(self.superuser)
        )

        # Test that own audit are manageable to permission 'taxsystem.manage_own_alliance'
        self.assertIn(
            self.alliance_audit,
            type(self.alliance_audit).objects.manage_to(self.manage_own_user),
        )
        self.assertNotIn(
            alliance_audit_2,
            type(alliance_audit_2).objects.manage_to(self.manage_own_user),
        )

        # Test that all alliance audits are manageable to permission 'taxsystem.manage_alliances'
        self.assertIn(
            alliance_audit_2, type(alliance_audit_2).objects.manage_to(self.manage_user)
        )
        self.assertIn(
            self.alliance_audit,
            type(self.alliance_audit).objects.manage_to(self.manage_user),
        )



# Generated by Django 4.2.26 on 2025-12-03 20:17

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0018_alter_allianceowner_corporation"),
    ]

    operations = [
        migrations.AlterField(
            model_name="alliancepayments",
            name="entry_id",
            field=models.BigIntegerField(blank=True, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="entry_id",
            field=models.BigIntegerField(blank=True, null=True, unique=True),
        ),
    ]



# Generated by Django 4.2.23 on 2025-08-11 23:51

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0007_alter_ownerupdatestatus_section"),
    ]

    operations = [
        migrations.CreateModel(
            name="JournalFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "filter_type",
                    models.CharField(
                        choices=[("reason", "Reason"), ("amount", "Amount")],
                        max_length=20,
                    ),
                ),
                ("value", models.CharField(max_length=255)),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="JournalFilterSet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, unique=True)),
                ("description", models.CharField(blank=True, max_length=255)),
                ("enabled", models.BooleanField(default=True)),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_filter_set",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Journal Filter Set",
                "verbose_name_plural": "Journal Filter Sets",
                "default_permissions": (),
            },
        ),
        migrations.DeleteModel(
            name="FilterAmount",
        ),
        migrations.DeleteModel(
            name="FilterDate",
        ),
        migrations.DeleteModel(
            name="FilterReason",
        ),
        migrations.RemoveField(
            model_name="smartgroup",
            name="filters",
        ),
        migrations.RemoveField(
            model_name="smartgroup",
            name="owner",
        ),
        migrations.DeleteModel(
            name="SmartFilter",
        ),
        migrations.DeleteModel(
            name="SmartGroup",
        ),
        migrations.AddField(
            model_name="journalfilter",
            name="filter_set",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_filters",
                to="taxsystem.journalfilterset",
            ),
        ),
        migrations.AddConstraint(
            model_name="journalfilter",
            constraint=models.UniqueConstraint(
                fields=("filter_set", "value"), name="unique_filter_value_per_set"
            ),
        ),
    ]



# Standard Library
import json

# Third Party
from ninja import NinjaAPI

# Django
from django.core.handlers.wsgi import WSGIRequest
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# AA TaxSystem
from taxsystem import __title__
from taxsystem.api.helpers import core
from taxsystem.api.helpers.icons import (
    get_members_delete_button,
)
from taxsystem.api.schema import (
    CharacterSchema,
    MembersSchema,
)
from taxsystem.forms import DeleteMemberForm
from taxsystem.helpers import lazy
from taxsystem.models.alliance import (
    AllianceOwner,
)
from taxsystem.models.corporation import (
    CorporationOwner,
    Members,
)
from taxsystem.models.helpers.textchoices import AdminActions
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)


class CorporationApiEndpoints:
    tags = ["Corporation Tax System"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "owner/{owner_id}/view/members/",
            response={200: list, 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_members(request, owner_id: int):
            """
            This Endpoint retrieves the members of the according Owner.

            Args:
                request (WSGIRequest): The HTTP request object.
                owner_id (int): The ID of the owner whose members are to be retrieved.
            Returns:
                MembersResponse: A response object containing the list of members.
            """
            # pylint: disable=duplicate-code
            owner, perms = core.get_manage_owner(request, owner_id)

            if owner is None:
                return 404, {"error": _("Owner not Found.")}

            if perms is False:
                return 403, {"error": _("Permission Denied.")}

            # Handle Alliance Members or Corporation Members
            if isinstance(owner, AllianceOwner):
                members = Members.objects.filter(
                    owner__eve_corporation__alliance__alliance_id=owner_id
                )
            else:
                members = (
                    Members.objects.filter(owner=owner)
                    .select_related("owner")
                    .order_by("character_name")
                )

            response_members_list: list[MembersSchema] = []
            for member in members:
                actions = ""
                # Create the delete button if member is missing and is Corporation Owner
                if perms and member.is_missing and isinstance(owner, CorporationOwner):
                    actions = get_members_delete_button(member=member)

                response_member = MembersSchema(
                    character=CharacterSchema(
                        character_id=member.character_id,
                        character_name=member.character_name,
                        character_portrait=lazy.get_character_portrait_url(
                            member.character_id, size=32, as_html=True
                        ),
                    ),
                    is_missing=member.is_missing,
                    is_noaccount=member.is_noaccount,
                    status=member.get_status_display(),
                    joined=member.joined,
                    actions=actions,
                )
                response_members_list.append(response_member)
            return response_members_list

        @api.post(
            "owner/{owner_id}/member/{member_pk}/manage/delete-member/",
            response={200: dict, 400: dict, 403: dict, 404: dict},
            tags=self.tags,
        )
        def delete_member(request: WSGIRequest, owner_id: int, member_pk: int):
            """
            Handle an Request to Delete a Member

            This Endpoint deletes a member from an associated tax account.
            It validates the request, checks permissions, and deletes the member from the according tax account.

            Args:
                request (WSGIRequest): The HTTP request object.
                owner_id (int): The ID of the owner whose filter set is to be retrieved.
                member_pk (int): The ID of the member to be deleted.
            Returns:
                dict: A dictionary containing the success status and message.
            """
            # pylint: disable=duplicate-code
            owner, perms = core.get_manage_owner(request, owner_id)

            # Check if owner exists
            if owner is None:
                return 404, {"error": _("Owner not Found.")}

            # Check permissions
            if perms is False:
                return 403, {"error": _("Permission Denied.")}

            # Validate the form data
            form = DeleteMemberForm(data=json.loads(request.body))
            if not form.is_valid():
                msg = _("Invalid form data.")
                return 400, {"success": False, "message": msg}

            reason = form.cleaned_data["comment"]

            member = Members.objects.get(owner=owner, pk=member_pk)
            if member.is_missing:
                msg = format_lazy(
                    _("Member {member} deleted - {reason}"),
                    member=member.character_name,
                    reason=reason,
                )
                member.delete()
                owner.admin_log_model(
                    user=request.user,
                    owner=owner,
                    action=AdminActions.DELETE,
                    comment=msg,
                ).save()
                return 200, {"success": True, "message": msg}
            msg = _("Member is not marked as missing.")
            return 400, {"success": False, "message": msg}



# Generated by Django 4.2.26 on 2025-12-16 12:47

# Django
import django.core.validators
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0019_alter_alliancepayments_entry_id_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation/Alliance"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                    ("manage_own_alliance", "Can manage own Alliance"),
                    ("manage_alliances", "Can manage all Alliances"),
                ),
            },
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="owner",
            field=models.ForeignKey(
                help_text="Owner that the action was performed on",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_admin_history",
                to="taxsystem.allianceowner",
                verbose_name="Owner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancefilter",
            name="value",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="active",
            field=models.BooleanField(
                default=True,
                help_text="Designates whether this alliance owner is active.",
            ),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="tax_amount",
            field=models.DecimalField(
                decimal_places=0,
                default=0,
                help_text="Tax Amount in ISK that is set for the alliance. Max 16 Digits",
                max_digits=16,
                validators=[django.core.validators.MaxValueValidator(9999999999999999)],
            ),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="tax_period",
            field=models.PositiveIntegerField(
                default=30,
                help_text="Tax Period in days for the alliance. Max 365 days. Default: 30 days",
                validators=[django.core.validators.MaxValueValidator(365)],
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_ally_tax_accounts",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Status Changed", "Status Changed"),
                    ("Payment Added", "Payment Added"),
                    ("Custom Payment Added", "Custom Payment Added"),
                    ("Reviser Comment", "Reviser Comment"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="owner",
            field=models.ForeignKey(
                help_text="Owner that the action was performed on",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_admin_history",
                to="taxsystem.corporationowner",
                verbose_name="Owner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationfilter",
            name="value",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="corporationowner",
            name="active",
            field=models.BooleanField(
                default=True,
                help_text="Designates whether this corporation owner is active.",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corp_tax_accounts",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Status Changed", "Status Changed"),
                    ("Payment Added", "Payment Added"),
                    ("Custom Payment Added", "Custom Payment Added"),
                    ("Reviser Comment", "Reviser Comment"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
    ]



"""Tests for admin.py"""

# Standard Library
from unittest.mock import Mock, patch

# Django
from django.contrib.admin.sites import AdminSite

# AA TaxSystem
from taxsystem.admin import CorporationOwnerAdmin
from taxsystem.models.corporation import CorporationOwner
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.utils import (
    create_owner_from_user,
)


class TestCorporationOwnerAdmin(TaxSystemTestCase):
    """Test Backend AA Administration for Tax System"""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.user.is_staff = True
        cls.user.is_superuser = True
        cls.user.save()

        cls.corporation_owner = create_owner_from_user(cls.user)
        cls.site = AdminSite()
        cls.admin = CorporationOwnerAdmin(CorporationOwner, cls.site)

    def test_list_display(self):
        """Test list_display configuration"""
        # given/when/then
        self.assertIn("_entity_pic", self.admin.list_display)
        self.assertIn("_eve_corporation__corporation_id", self.admin.list_display)
        self.assertIn("_eve_corporation__corporation_name", self.admin.list_display)
        self.assertIn("_last_update_at", self.admin.list_display)

    def test_entity_pic_display(self):
        """Test _entity_pic method returns HTML"""
        # given/when
        result = self.admin._entity_pic(self.corporation_owner)
        # then
        self.assertIn("<img src=", result)
        self.assertIn('class="img-circle"', result)

    def test_corporation_id_display(self):
        """Test _eve_corporation__corporation_id method"""
        # given/when
        result = self.admin._eve_corporation__corporation_id(self.corporation_owner)
        # then
        self.assertEqual(result, self.corporation_owner.eve_corporation.corporation_id)

    def test_corporation_name_display(self):
        """Test _eve_corporation__corporation_name method"""
        # given/when
        result = self.admin._eve_corporation__corporation_name(self.corporation_owner)
        # then
        self.assertEqual(
            result, self.corporation_owner.eve_corporation.corporation_name
        )

    def test_last_update_at_display_with_data(self):
        """Test _last_update_at method with update status"""
        # given
        self.corporation_owner.last_update_at = "2025-11-22 10:00:00"
        # when
        result = self.admin._last_update_at(self.corporation_owner)
        # then
        self.assertIsNotNone(result)
        self.assertNotEqual(result, "-")

    def test_last_update_at_display_without_data(self):
        """Test _last_update_at method without update status"""
        # given
        self.corporation_owner.last_update_at = None
        # when
        result = self.admin._last_update_at(self.corporation_owner)
        # then
        self.assertEqual(result, "-")

    def test_has_add_permission_returns_false(self):
        """Test has_add_permission always returns False"""
        # given
        request = self.factory.get("/admin/taxsystem/corporationowner/")
        request.user = self.user
        # when
        result = self.admin.has_add_permission(request)
        # then
        self.assertFalse(result)

    def test_has_change_permission_returns_false(self):
        """Test has_change_permission always returns False"""
        # given
        request = self.factory.get("/admin/taxsystem/corporationowner/")
        request.user = self.user
        # when
        result = self.admin.has_change_permission(request)
        # then
        self.assertFalse(result)

    @patch("taxsystem.admin.update_corporation")
    def test_force_update_action(self, mock_update):
        """Test force_update action"""
        # given
        request = self.factory.post("/admin/taxsystem/corporationowner/")
        request.user = self.user
        request._messages = Mock()
        queryset = CorporationOwner.objects.filter(pk=self.corporation_owner.pk)
        # when
        self.admin.force_update(request, queryset)
        # then
        mock_update.delay.assert_called_once_with(
            owner_eve_id=self.corporation_owner.eve_id, force_refresh=True
        )



# Standard Library
from unittest.mock import MagicMock, patch

# Django
from django.test import override_settings

# AA TaxSystem
from taxsystem.models.general import EveEntity
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.esi_stub_openapi import (
    EsiEndpoint,
    create_esi_client_stub,
)
from taxsystem.tests.testdata.utils import (
    create_division,
    create_owner_from_user,
)

MODULE_PATH = "taxsystem.managers.wallet_manager"

TAXSYSTEM_WALLET_JOURNAL_ENDPOINTS = [
    EsiEndpoint(
        category="Wallet",
        method="GetCorporationsCorporationIdWalletsDivisionJournal",
        param_names=("corporation_id", "division"),
    ),
    EsiEndpoint(
        category="Corporation",
        method="GetCorporationsCorporationIdDivisions",
        param_names="corporation_id",
    ),
    EsiEndpoint(
        category="Wallet",
        method="GetCorporationsCorporationIdWallets",
        param_names="corporation_id",
    ),
]


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
@patch(MODULE_PATH + ".esi")
@patch(MODULE_PATH + ".EveEntity.objects.bulk_resolve_names")
@patch(MODULE_PATH + ".EveEntity.objects.filter")
class TestWalletManager(TaxSystemTestCase):
    """Test Wallet Managers for Corporation."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.audit = create_owner_from_user(cls.user)

        cls.eve_character_first_party = EveEntity.objects.get(id=1002)
        cls.eve_character_second_party = EveEntity.objects.get(id=1001)

        cls.division = create_division(
            corporation=cls.audit, name="MEGA KONTO", balance=1000000, division_id=1
        )
        cls.token = cls.user_character.user.token_set.first()
        cls.audit.get_token = MagicMock(return_value=cls.token)

    def test_update_wallet_journal(self, mock_filter, mock_entity_bulk, mock_esi):
        """
        Test updating wallet journal entries from ESI data.
        This test should verify that wallet journal entries are correctly fetched and stored.

        Results:
            1. Wallet Journal Entries (entry_id: 10, 13, 16) are created with correct data.
            2. First and Second party entities are resolved correctly.
            3. Amounts and context IDs are stored accurately.
        """
        # Test Data
        mock_esi.client = create_esi_client_stub(
            endpoints=TAXSYSTEM_WALLET_JOURNAL_ENDPOINTS
        )
        filter_mock = mock_filter.return_value
        filter_mock.count.return_value = 0

        mock_entity_bulk.side_effect = [
            EveEntity.objects.create(
                id=9998,
                name="Test Character",
                category="character",
            ),
        ]

        # Test Action

        self.audit.update_wallet(force_refresh=False)

        # Expected Results
        self.assertSetEqual(
            set(self.division.ts_corporation_wallet.values_list("entry_id", flat=True)),
            {10, 13, 16},
        )
        obj = self.division.ts_corporation_wallet.get(entry_id=10)
        self.assertEqual(obj.amount, 1000)
        self.assertEqual(obj.context_id, 1)
        self.assertEqual(obj.first_party.id, 2001)
        self.assertEqual(obj.second_party.id, 1001)

        obj = self.division.ts_corporation_wallet.get(entry_id=13)
        self.assertEqual(obj.amount, 5000)

        obj = self.division.ts_corporation_wallet.get(entry_id=16)
        self.assertEqual(obj.amount, 10000)

    def test_update_division_names(self, mock_filter, mock_entity_bulk, mock_esi):
        """
        Test updating division names from ESI data.
        This test should verify that division names are correctly fetched and updated.

        Results:
            1. Division names are updated correctly based on ESI data.
        """
        # Test Data
        mock_esi.client = create_esi_client_stub(
            endpoints=TAXSYSTEM_WALLET_JOURNAL_ENDPOINTS
        )

        # Test Action
        self.audit.update_division_names(force_refresh=False)

        # Expected Results
        obj = self.audit.ts_corporation_division.get(
            corporation__eve_corporation__corporation_id=2001, division_id=2
        )
        self.assertEqual(obj.name, "Rechnungen")

        obj = self.audit.ts_corporation_division.get(
            corporation__eve_corporation__corporation_id=2001, division_id=4
        )
        self.assertEqual(obj.name, "Ship Replacment Abteilung")

        obj = self.audit.ts_corporation_division.get(
            corporation__eve_corporation__corporation_id=2001, division_id=6
        )
        self.assertEqual(obj.name, "Partner")

    def test_update_divisions(self, mock_filter, mock_entity_bulk, mock_esi):
        """
        Test updating division balances from ESI data.
        This test should verify that division balances are correctly fetched and updated.

        Results:
            1. Division balances are updated correctly based on ESI data.
        """
        # Test Data
        mock_esi.client = create_esi_client_stub(
            endpoints=TAXSYSTEM_WALLET_JOURNAL_ENDPOINTS
        )

        # Test Action
        self.audit.update_divisions(force_refresh=False)

        # Expected Results
        obj = self.audit.ts_corporation_division.get(
            corporation__eve_corporation__corporation_id=2001, division_id=2
        )
        self.assertEqual(obj.balance, 0)

        obj = self.audit.ts_corporation_division.get(
            corporation__eve_corporation__corporation_id=2001, division_id=4
        )
        self.assertEqual(obj.balance, 500000)

        obj = self.audit.ts_corporation_division.get(
            corporation__eve_corporation__corporation_id=2001, division_id=6
        )
        self.assertEqual(obj.balance, 250000)



"""Models for Tax System."""

# Django
from django.core.validators import MaxValueValidator
from django.db import models
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.eveonline.models import (
    EveAllianceInfo,
)
from allianceauth.services.hooks import get_extension_logger

# AA TaxSystem
from taxsystem import __title__
from taxsystem.managers.alliance_manager import (
    AlliancePaymentAccountManager,
    AlliancePaymentManager,
)
from taxsystem.managers.owner_manager import AllianceOwnerManager
from taxsystem.models.base import (
    FilterBaseModel,
    FilterSetBaseModel,
    HistoryBaseModel,
    PaymentAccountBaseModel,
    PaymentsBaseModel,
    UpdateStatusBaseModel,
)
from taxsystem.models.corporation import CorporationOwner
from taxsystem.models.general import UpdateSectionResult
from taxsystem.models.helpers.textchoices import (
    AdminActions,
    AllianceUpdateSection,
    PaymentRequestStatus,
    UpdateStatus,
)
from taxsystem.models.helpers.updater import UpdateManager
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)


class AllianceUpdateStatus(UpdateStatusBaseModel):
    """Model representing the update status of an alliance owner in the tax system."""

    owner = models.ForeignKey(
        "AllianceOwner",
        on_delete=models.CASCADE,
        related_name="ts_alliance_update_status",
    )

    section = models.CharField(
        max_length=32, choices=AllianceUpdateSection.choices, db_index=True
    )

    class Meta:
        default_permissions = ()
        unique_together = [("owner", "section")]

    def __str__(self) -> str:
        return f"{self.owner.name} - {self.section}"


class AllianceOwner(models.Model):
    """Model representing an alliance owner in the tax system."""

    class Meta:
        default_permissions = ()

    objects: AllianceOwnerManager = AllianceOwnerManager()

    name = models.CharField(
        max_length=255,
    )

    eve_alliance = models.OneToOneField(
        EveAllianceInfo,
        on_delete=models.CASCADE,
        related_name="+",
    )

    corporation = models.ForeignKey(
        CorporationOwner,
        on_delete=models.PROTECT,
        related_name="+",
        help_text=_("The corporation that owns this alliance tax system."),
    )

    active = models.BooleanField(
        default=True,
        help_text=_("Designates whether this alliance owner is active."),
    )

    tax_amount = models.DecimalField(
        max_digits=16,
        decimal_places=0,
        help_text=_("Tax Amount in ISK that is set for the alliance. Max 16 Digits"),
        default=0,
        validators=[MaxValueValidator(9999999999999999)],
    )

    tax_period = models.PositiveIntegerField(
        help_text=_(
            "Tax Period in days for the alliance. Max 365 days. Default: 30 days"
        ),
        default=30,
        validators=[MaxValueValidator(365)],
    )

    def __str__(self) -> str:
        return f"{self.eve_alliance.alliance_name}"

    @property
    def eve_id(self) -> int:
        """Return the Eve Alliance ID."""
        return self.eve_alliance.alliance_id

    def update_payments(self, force_refresh: bool) -> UpdateSectionResult:
        """Update the payments for this owner.
        Args:
            force_refresh: Force refresh
        Returns:
            UpdateSectionResult object for this section
        """
        return AlliancePayments.objects.update_or_create_payments(
            owner=self, force_refresh=force_refresh
        )

    def update_tax_accounts(self, force_refresh: bool) -> UpdateSectionResult:
        """Update the tax accounts for this owner.
        Args:
            force_refresh: Force refresh
        Returns:
            UpdateSectionResult object for this section
        """
        return AlliancePaymentAccount.objects.update_or_create_tax_accounts(
            owner=self, force_refresh=force_refresh
        )

    def update_deadlines(self, force_refresh: bool) -> UpdateSectionResult:
        """Update the tax deadlines for this owner.
        Args:
            force_refresh: Force refresh
        Returns:
            UpdateSectionResult object for this section
        """
        return AlliancePaymentAccount.objects.check_payment_deadlines(
            owner=self, force_refresh=force_refresh
        )

    @property
    def payment_model(self):
        """Return the Payment Model for this owner."""
        return AlliancePayments

    @property
    def payment_history_model(self):
        """Return the Payment History Model for this owner."""
        return AlliancePaymentHistory

    @property
    def admin_log_model(self):
        """Return the Admin History Model for this owner."""
        return AllianceAdminHistory

    @property
    def account_model(self):
        """Return the Tax Account Model for this owner."""
        return AlliancePaymentAccount

    @property
    def filterset_model(self):
        """Return the Filter Set Model for this owner."""
        return AllianceFilterSet

    @property
    def filter_model(self):
        """Return the Filter Model for this owner."""
        return AllianceFilter

    @property
    def update_manager(self):
        """Return the Update Manager helper for this owner."""
        return UpdateManager(
            owner=self,
            update_section=AllianceUpdateSection,
            update_status=AllianceUpdateStatus,
        )

    @property
    def get_status(self) -> UpdateStatus:
        """Get the update status of this owner.

        Returns:
            UpdateStatus enum value representing the current status
        """
        # pylint: disable=duplicate-code
        if self.active is False:
            return UpdateStatus.DISABLED

        # Use type(self) for dynamic QuerySet resolution
        qs = type(self).objects.filter(pk=self.pk).annotate_total_update_status()
        total_update_status = list(qs.values_list("total_update_status", flat=True))[0]
        return UpdateStatus(total_update_status)

    @property
    def get_update_status(self) -> dict[str, str]:
        """Return a dictionary of update sections and their statuses."""
        update_status = {}
        for section in AllianceUpdateSection.get_sections():
            try:
                status = AllianceUpdateStatus.objects.get(owner=self, section=section)
                update_status[section] = {
                    "is_success": status.is_success,
                    "last_update_finished_at": status.last_update_finished_at,
                    "last_run_finished_at": status.last_run_finished_at,
                }
            except AllianceUpdateStatus.DoesNotExist:
                continue
        return update_status


class AlliancePaymentAccount(PaymentAccountBaseModel):
    """Model representing an alliance tax account in the tax system."""

    objects: AlliancePaymentAccountManager = AlliancePaymentAccountManager()

    class Meta:
        default_permissions = ()

    owner = models.ForeignKey(
        AllianceOwner,
        on_delete=models.CASCADE,
        related_name="ts_ally_tax_accounts",
    )


class AlliancePayments(PaymentsBaseModel):
    """Model representing payments made by alliance members in the tax system."""

    objects: AlliancePaymentManager = AlliancePaymentManager()

    class Meta:
        default_permissions = ()

    account = models.ForeignKey(
        AlliancePaymentAccount,
        on_delete=models.CASCADE,
        related_name="+",
    )

    owner = models.ForeignKey(
        AllianceOwner,
        on_delete=models.CASCADE,
        related_name="ts_alliance_payments",
        help_text=_("Owner of this payment"),
        null=True,
        blank=True,
    )

    def __str__(self) -> str:
        return f"{self.account.name} - {self.amount} ISK"

    def transaction_log(
        self, user, comment, new_status, action=""
    ) -> "AlliancePaymentHistory":
        """Return a log entry for the transaction.

        Args:
            user: User performing the action
            comment: Additional message
            new_status: New status after the action
            action: Action performed (optional)
        Returns:
            AlliancePaymentHistory object
        """
        return AlliancePaymentHistory(
            user=user,
            payment=self,
            new_status=new_status,
            action=action,
            comment=comment,
        )


class AllianceFilterSet(FilterSetBaseModel):
    owner = models.ForeignKey(
        AllianceOwner,
        on_delete=models.CASCADE,
        related_name="ts_alliance_filter_set",
    )

    def filter(
        self, payments: models.QuerySet[AlliancePayments]
    ) -> models.QuerySet[AlliancePayments]:
        """Apply filters to the given payments queryset."""
        if self.is_active:
            queries = []
            for f in self.ts_alliance_filters.all():
                f: AllianceFilter
                # Generate Q object for each filter
                q = f.get_match_type_filter()
                if q is not None:
                    queries.append(q)

            # If no queries were generated, return empty queryset
            if not queries:
                return AlliancePayments.objects.none()

            # Combine all queries using AND operation
            combined = queries.pop()
            for q in queries:
                combined &= q
            return payments.filter(combined)
        return AlliancePayments.objects.none()

    def __str__(self) -> str:
        return f"Filter Set: {self.name}"


class AllianceFilter(FilterBaseModel):
    class Meta:
        default_permissions = ()

    filter_set = models.ForeignKey(
        AllianceFilterSet,
        on_delete=models.CASCADE,
        related_name="ts_alliance_filters",
    )


class AlliancePaymentHistory(HistoryBaseModel):
    """Model representing the history of actions taken on alliance payments in the tax system."""

    class Meta:
        default_permissions = ()

    # pylint: disable=duplicate-code
    payment = models.ForeignKey(
        AlliancePayments,
        on_delete=models.CASCADE,
        related_name="+",
        verbose_name=_("Payment"),
        help_text=_("Payment that the action was performed on"),
    )
    # pylint: enable=duplicate-code
    new_status = models.CharField(
        max_length=20,
        choices=PaymentRequestStatus.choices,
        verbose_name=_("New Status"),
        help_text=_("New Status of the action"),
    )


class AllianceAdminHistory(HistoryBaseModel):
    """
    Model representing the history of administrative actions taken on owners in the tax system.
    """

    class Meta:
        default_permissions = ()

    owner = models.ForeignKey(
        AllianceOwner,
        verbose_name=_("Owner"),
        help_text=_("Owner that the action was performed on"),
        on_delete=models.CASCADE,
        related_name="ts_admin_history",
    )

    action = models.CharField(
        max_length=20,
        choices=AdminActions.choices,
        default=AdminActions.DEFAULT,
        verbose_name=_("Action"),
        help_text=_("Action performed"),
    )



# Django
from django.urls import reverse
from django.utils import timezone

# AA TaxSystem
from taxsystem.api.helpers.icons import get_taxsystem_payments_action_icons
from taxsystem.models.general import EveEntity
from taxsystem.models.helpers.textchoices import PaymentRequestStatus
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.utils import (
    create_division,
    create_owner_from_user,
    create_payment,
    create_tax_account,
    create_wallet_journal_entry,
)

MODULE_PATH = "taxsystem.api.helpers."
API_URL = "taxsystem:api"


class TestPaymentsApiEndpoints(TaxSystemTestCase):
    """Test Payments API Endpoints."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.audit = create_owner_from_user(user=cls.user)

        cls.division = create_division(
            corporation=cls.audit, balance=0, division_id=1, name="Main Division"
        )

        cls.first_party = EveEntity.objects.get(id=1002)
        cls.second_party = EveEntity.objects.get(id=1001)

        cls.tax_account = create_tax_account(
            name=cls.user_character.character.character_name,
            owner=cls.audit,
            user=cls.user,
            status="active",
            deposit=0,
        )

    def test_get_taxsystem_payments_action_icons(self):
        """
        Test get_taxsystem_payments_action_icons function.

        Create a payment with different actions and verify the returned icons.

        # Test Szenarios:
            - Display approve and reject icons for pending payment.
            - Display Undo icon for approved payment.
            - Display Delete icon for non ESI payment.
        """
        # Test Data
        request = self.factory.get(reverse("taxsystem:index"))
        request.user = self.manage_user

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        # Pending Payment
        payment = create_payment(
            name=self.user_character.character.character_name,
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        payment2 = create_payment(
            name="Second Payment",
            account=self.tax_account,
            owner=self.audit,
            journal=None,
            amount=500,
            date=timezone.datetime(2025, 1, 2, 12, 0, 0),
            reason="Tax Payment 2",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        # Test Action
        icons = get_taxsystem_payments_action_icons(request=request, payment=payment)

        # Exptected Results
        approve_icon = '<i class="fa-solid fa-check"></i>'
        reject_icon = '<i class="fa-solid fa-xmark"></i>'
        undo_icon = '<i class="fa-solid fa-undo"></i>'
        delete_icon = '<i class="fa-solid fa-trash"></i>'

        self.assertIn(approve_icon, icons)
        self.assertIn(reject_icon, icons)
        self.assertNotIn(undo_icon, icons)
        self.assertNotIn(delete_icon, icons)

        # Test Undo Icon for Approved Payment
        payment.request_status = PaymentRequestStatus.APPROVED
        payment.save()

        # Test Action
        icons = get_taxsystem_payments_action_icons(request=request, payment=payment)

        # Exptected Result
        self.assertIn(undo_icon, icons)

        # Test Action for Non ESI Payment
        icons = get_taxsystem_payments_action_icons(request=request, payment=payment2)

        # Exptected Result
        self.assertIn(delete_icon, icons)



# Standard Library
import json
from http import HTTPStatus

# Django
from django.urls import reverse

# AA TaxSystem
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.utils import create_member, create_owner_from_user

MODULE_PATH = "taxsystem.api.helpers."
API_URL = "taxsystem:api"


class TestCorporationApiEndpoints(TaxSystemTestCase):
    """Test Corporation API Endpoints."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.audit = create_owner_from_user(user=cls.user)

    def test_get_members_should_403(self):
        """
        Test should return 403 Forbidden when user lacks permissions.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id
        url = reverse(f"{API_URL}:get_members", kwargs={"owner_id": corporation_id})
        self.client.force_login(self.user)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)

    def test_get_members_should_404(self):
        """
        Test should return 404 Not Found when resource does not exist.
        """
        # Test Data
        url = reverse(f"{API_URL}:get_members", kwargs={"owner_id": 9999})
        self.client.force_login(self.superuser)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.NOT_FOUND)

    def test_get_members_should_200(self):
        """
        Test should return 200 OK when user has permissions.

        Results:
        - Contains Test Character
        - Contains Missing Character
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id
        create_member(
            owner=self.audit,
            character_id=9999,
            character_name="Test Character",
            joined="2023-01-01",
            status="active",
        )
        create_member(
            owner=self.audit,
            character_id=10000,
            character_name="Missing Character",
            joined="2022-01-01",
            status="missing",
        )

        url = reverse(f"{API_URL}:get_members", kwargs={"owner_id": corporation_id})
        self.client.force_login(self.superuser)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertIn("Test Character", str(response.json()))
        self.assertIn("Missing Character", str(response.json()))

    def test_delete_member_should_403(self):
        """
        Test should return 403 Forbidden when user lacks permissions.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id
        member = create_member(
            owner=self.audit,
            character_id=9999,
            character_name="Test Character",
        )
        url = reverse(
            f"{API_URL}:delete_member",
            kwargs={"owner_id": corporation_id, "member_pk": member.pk},
        )
        data = {"comment": "Removing member for testing purposes."}
        self.client.force_login(self.user)

        # Test Action
        response = self.client.post(path=url, data=data)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)

    def test_delete_member_should_200(self):
        """
        Test should return 200 OK when user has permissions.

        Results:
        - Member is deleted
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id
        member = create_member(
            owner=self.audit,
            character_id=9999,
            character_name="Test Character",
            status="missing",
        )
        url = reverse(
            f"{API_URL}:delete_member",
            kwargs={"owner_id": corporation_id, "member_pk": member.pk},
        )
        data = {"comment": "Removing member for testing purposes."}
        self.client.force_login(self.superuser)

        # Test Action
        response = self.client.post(
            path=url, body=json.dumps(data), content_type="application/json"
        )
        print(json.loads(response.content))

        # Expected Result
        result = "Member {member} deleted - {reason}".format(
            member=member.character_name, reason=data["comment"]
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertIn(response.json().get("message"), result)



# Generated by Django 4.2.26 on 2025-12-23 01:03

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "taxsystem",
            "0021_remove_alliancepayments_taxsystem_a_account_677604_idx_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "added"),
                    ("Changed", "changed"),
                    ("Deleted", "deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "added"),
                    ("Changed", "changed"),
                    ("Deleted", "deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "active"),
                    ("inactive", "inactive"),
                    ("deactivated", "deactivated"),
                    ("missing", "missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "active"),
                    ("inactive", "inactive"),
                    ("deactivated", "deactivated"),
                    ("missing", "missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="members",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "active"),
                    ("missing", "missing"),
                    ("noaccount", "unregistered"),
                    ("is_alt", "is alt"),
                ],
                default="active",
                max_length=10,
                verbose_name="Status",
            ),
        ),
    ]



# Django
from django.core.management.base import BaseCommand
from django.db import IntegrityError, transaction

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# AA TaxSystem
from taxsystem import __title__
from taxsystem.models.corporation import CorporationOwner, CorporationPayments
from taxsystem.models.wallet import CorporationWalletJournalEntry
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)


class Command(BaseCommand):
    help = "Migrate Corporations to new Payments Model"

    # pylint: disable=unused-argument
    def handle(self, *args, **options):
        corporations = CorporationOwner.objects.all()
        payments_entry_ids = CorporationPayments.objects.all().values_list(
            "entry_id", flat=True
        )
        if not corporations:
            self.stdout.write(
                "No Corporations found in the database. Skipping migration."
            )
            return

        for corporation in corporations:
            try:
                with transaction.atomic():
                    journals = CorporationWalletJournalEntry.objects.filter(
                        division__corporation=corporation
                    ).select_related("division")

                    if not journals:
                        self.stdout.write(
                            f"No Wallet Divisions found for {corporation}. Skipping..."
                        )
                        continue

                    successful = 0
                    for journal in journals:
                        if journal.entry_id in payments_entry_ids:
                            try:
                                payment = CorporationPayments.objects.get(
                                    owner__isnull=True,
                                    entry_id=journal.entry_id,
                                )
                                payment.owner = corporation
                                payment.journal = journal
                                payment.save()
                                self.stdout.write(
                                    f"Updated Payment {payment.pk} and assigned to {corporation} for entry_id {journal.entry_id}."
                                )
                                successful += 1
                                continue
                            except CorporationPayments.DoesNotExist:
                                self.stdout.write(
                                    f"Payment with entry_id {journal.entry_id} not found, skipping."
                                )
                                continue
                            except CorporationPayments.MultipleObjectsReturned:
                                # Inform user to run cleanup command to resolve duplicates
                                self.stdout.write(
                                    f"Multiple payments found for entry_id {journal.entry_id}. "
                                    "Please run: `python manage.py taxsystem_cleanup_payments` "
                                    "to remove duplicate payments and keep the oldest entry."
                                )
                                continue
                    self.stdout.write(
                        f"Migration report for {corporation}: {successful} entries migrated."
                    )
            except IntegrityError as e:
                self.stdout.write(f"Failed to create Payment for {corporation}: {e}")
                continue



"""Initialize the app"""

__version__ = "3.0.3"
__title__ = "Tax System"

__package_name__ = "aa-taxsystem"
__app_name__ = "taxsystem"
__esi_compatibility_date__ = "2025-12-16"
__app_name_useragent__ = "AA-TaxSystem"

__github_url__ = f"https://github.com/Geuthur/{__package_name__}"

__operations__ = [
    "GetCorporationsCorporationIdWallets",
    "GetCorporationsCorporationIdWalletsDivisionJournal",
    "GetCorporationsCorporationIdDivisions",
    "GetCorporationsCorporationIdMembertracking",
    # Character operations
    "GetCharactersCharacterIdRoles",
    # Universe operations
    "PostUniverseNames",
]



# Django
from django.utils import timezone

# AA TaxSystem
from taxsystem.models.corporation import CorporationPaymentAccount
from taxsystem.models.helpers.textchoices import AccountStatus
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.utils import (
    create_owner_from_user,
    create_tax_account,
)

MODULE_PATH = "taxsystem.models.tax"


class TestPaymentSystemModel(TaxSystemTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.audit = create_owner_from_user(cls.user)
        cls.audit2 = create_owner_from_user(cls.superuser)

        cls.tax_account = create_tax_account(
            name=cls.user.username,
            owner=cls.audit,
            user=cls.user,
            deposit=0,
        )

    def test_str(self):
        expected_str = CorporationPaymentAccount.objects.get(owner=self.audit)
        self.assertEqual(self.tax_account, expected_str)

    def test_is_active(self):
        """Test if the tax account is active."""
        tax_account = CorporationPaymentAccount.objects.get(owner=self.audit)
        self.assertTrue(tax_account.is_active)

    def test_is_inactive(self):
        """Test if the tax account is inactive."""
        self.tax_account.status = AccountStatus.INACTIVE
        self.tax_account.save()

        tax_account = CorporationPaymentAccount.objects.get(owner=self.audit)
        self.assertFalse(tax_account.is_active)

    def test_is_deactivated(self):
        """Test if the tax account is deactivated."""
        self.tax_account.status = AccountStatus.DEACTIVATED
        self.tax_account.save()
        tax_account = CorporationPaymentAccount.objects.get(owner=self.audit)
        self.assertFalse(tax_account.is_active)

    def test_is_missing(self):
        """Test if the tax account is missing."""
        self.tax_account.status = AccountStatus.MISSING
        self.tax_account.save()

        tax_account = CorporationPaymentAccount.objects.get(owner=self.audit)
        self.assertFalse(tax_account.is_active)

    def test_has_paid(self):
        """Test if the tax account has paid."""
        self.tax_account.deposit = 1000
        self.tax_account.save()

        self.tax_account.date = timezone.now()

        tax_account = CorporationPaymentAccount.objects.get(owner=self.audit)
        self.assertTrue(tax_account.has_paid)

    def test_has_paid_icon(self):
        """Test the icon representation of has_paid."""
        self.tax_account.deposit = 1000
        self.tax_account.save()

        self.tax_account.date = timezone.now()

        tax_account = CorporationPaymentAccount.objects.get(owner=self.audit)

        self.assertIn(
            "fas fa-check",
            tax_account.has_paid_icon(),
        )
        self.assertIn(
            "badge",
            tax_account.has_paid_icon(badge=True, text=True),
        )
        self.assertIn(
            "Paid",
            tax_account.has_paid_icon(badge=True, text=True),
        )
        self.assertNotIn(
            "badge",
            tax_account.has_paid_icon(badge=False, text=True),
        )

    def test_status_html(self):
        """Test the HTML representation of the tax account status."""
        tax_account = CorporationPaymentAccount.objects.get(owner=self.audit)
        self.assertIn(
            "bg-success",
            AccountStatus(tax_account.status).html(),
        )

        self.assertIn(
            "Active",
            AccountStatus(tax_account.status).html(text=True),
        )

    def test_status_color(self):
        """Test the color representation of the tax account status."""
        tax_account = CorporationPaymentAccount.objects.get(owner=self.audit)
        self.assertEqual(AccountStatus(tax_account.status).color(), "success")

    def test_status_icon(self):
        """Test the icon representation of the tax account status."""
        tax_account = CorporationPaymentAccount.objects.get(owner=self.audit)

        self.assertEqual(
            AccountStatus(tax_account.status).icon(),
            "<i class='fas fa-check'></i>",
        )



# Generated by Django 4.2.28 on 2026-02-26 07:19

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0025_eveentity_and_more"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="corporationwalletjournalentry",
            name="taxsystem_c_first_p_d83db8_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationwalletjournalentry",
            name="taxsystem_c_second__2a759e_idx",
        ),
        migrations.RemoveField(
            model_name="corporationwalletjournalentry",
            name="first_party",
        ),
        migrations.RemoveField(
            model_name="corporationwalletjournalentry",
            name="second_party",
        ),
    ]



# Third Party
from ninja import NinjaAPI, Schema

# Django
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# AA TaxSystem
from taxsystem import __title__
from taxsystem.api.helpers import core
from taxsystem.api.schema import (
    AdminHistorySchema,
    PaymentHistorySchema,
)
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)


class AdminLogResponse(Schema):
    owner: list[AdminHistorySchema]


class PaymentsLogResponse(Schema):
    owner: list[AdminHistorySchema]


class LogsApiEndpoints:
    tags = ["Logs"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "owner/admin/{owner_id}/view/logs/",
            response={200: PaymentsLogResponse, 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_payments_history(request, owner_id: int):
            """
            This Endpoint retrieves the payments logs associated with a specific owner.

            Args:
                request (WSGIRequest): The HTTP request object.
                owner_id (int): The ID of the owner whose payments logs are to be retrieved.
            Returns:
                PaymentsLogResponse: A response object containing the list of payments logs.
            """
            # pylint: disable=duplicate-code
            owner, perms = core.get_manage_owner(request, owner_id)

            if owner is None:
                return 404, {"error": _("Owner not Found.")}

            if perms is False:
                return 403, {"error": _("Permission Denied.")}

            logs = (
                owner.admin_log_model.objects.filter(owner=owner)
                .select_related("user")
                .order_by("-date")
            )

            # Use generic helper function
            response_admin_logs_list: list[PaymentHistorySchema] = []
            for log in logs:
                response_log = PaymentHistorySchema(
                    log_id=log.pk,
                    user_name=log.user.username,
                    date=timezone.localtime(log.date).strftime("%Y-%m-%d %H:%M"),
                    action=log.action,
                    comment=log.comment,
                )
                response_admin_logs_list.append(response_log)

            return PaymentsLogResponse(owner=response_admin_logs_list)

        @api.get(
            "owner/admin/{owner_id}/view/logs/",
            response={200: AdminLogResponse, 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_owner_admin_logs(request, owner_id: int):
            """
            This Endpoint retrieves the admin logs associated with a specific owner.

            Args:
                request (WSGIRequest): The HTTP request object.
                owner_id (int): The ID of the owner whose admin logs are to be retrieved.
            Returns:
                AdminLogResponse: A response object containing the list of admin logs.
            """
            # pylint: disable=duplicate-code
            owner, perms = core.get_manage_owner(request, owner_id)

            if owner is None:
                return 404, {"error": _("Owner not Found.")}

            if perms is False:
                return 403, {"error": _("Permission Denied.")}

            logs = (
                owner.admin_log_model.objects.filter(owner=owner)
                .select_related("user")
                .order_by("-date")
            )

            # Use generic helper function
            response_admin_logs_list: list[AdminHistorySchema] = []
            for log in logs:
                response_log = AdminHistorySchema(
                    log_id=log.pk,
                    user_name=log.user.username,
                    date=timezone.localtime(log.date).strftime("%Y-%m-%d %H:%M"),
                    action=log.action,
                    comment=log.comment,
                )
                response_admin_logs_list.append(response_log)

            return AdminLogResponse(owner=response_admin_logs_list)



# Standard Library
from unittest.mock import patch

# Django
from django.test import override_settings
from django.utils import timezone

# AA TaxSystem
from taxsystem.models.corporation import (
    CorporationFilter,
    CorporationPaymentAccount,
    CorporationPayments,
    Members,
)
from taxsystem.models.general import EveEntity
from taxsystem.models.helpers.textchoices import AccountStatus, PaymentRequestStatus
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.esi_stub_openapi import (
    EsiEndpoint,
    create_esi_client_stub,
)
from taxsystem.tests.testdata.utils import (
    create_division,
    create_filter,
    create_filterset,
    create_member,
    create_owner_from_user,
    create_payment,
    create_tax_account,
    create_user_from_evecharacter,
    create_wallet_journal_entry,
)

MODULE_PATH = "taxsystem.managers.corporation_manager"

TEST_CORPORATION_MANAGER_ENDPOINTS = [
    EsiEndpoint(
        "Corporation", "GetCorporationsCorporationIdMembertracking", "corporation_id"
    ),
]


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestCorporationManager(TaxSystemTestCase):
    """Test Corporation Managers."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.audit = create_owner_from_user(cls.user)

        cls.filter_set = create_filterset(
            owner=cls.audit,
            name="100m",
            description="Filter for payments over 100m",
        )

        cls.filter_amount = create_filter(
            filter_set=cls.filter_set,
            filter_type=CorporationFilter.FilterType.AMOUNT,
            value=1000,
        )

        cls.division = create_division(
            corporation=cls.audit,
            division_id=1,
            name="Main Division",
            balance=1000000,
        )

        cls.eve_character_first_party = EveEntity.objects.get(id=1002)
        cls.eve_character_second_party = EveEntity.objects.get(id=1001)

    def test_update_tax_account(self):
        """
        Test updating corporation tax accounts payments.
        This test should change 2 Payments in the payment system depending on the given filters.

        Results:
            1. Approve a payment as APPROVED depending to the filter.
            2. Mark a payment as NEEDS_APPROVAL.
        """
        # Test Data
        tax_account = create_tax_account(
            name=self.user_character.character.character_name,
            owner=self.audit,
            user=self.user,
            status=AccountStatus.ACTIVE,
            deposit=0,
            last_paid=(timezone.now() - timezone.timedelta(days=30)),
        )

        journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.eve_character_first_party,
            second_party=self.eve_character_second_party,
            description="Test Description",
        )

        journal_entry2 = create_wallet_journal_entry(
            division=self.division,
            entry_id=2,
            amount=6000,
            date=timezone.datetime(2025, 1, 1, 14, 0, 0),
            reason="Mining Stuff",
            ref_type="tax_payment",
            first_party=self.eve_character_first_party,
            second_party=self.eve_character_second_party,
            description="Test Description 2",
        )

        # Approved Payment
        create_payment(
            name=self.user_character.character.character_name,
            account=tax_account,
            owner=self.audit,
            journal=journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        # Needs Approval Payment
        create_payment(
            name=self.user_character.character.character_name,
            account=tax_account,
            owner=self.audit,
            journal=journal_entry2,
            amount=6000,
            date=timezone.datetime(2025, 1, 1, 14, 0, 0),
            reason="Mining Stuff",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )
        print("before: %s", self.audit.ts_corporation_payments)
        # Test Action
        self.audit.update_tax_accounts(force_refresh=False)
        print("after: %s", self.audit.ts_corporation_payments)
        # Expected Results
        self.assertSetEqual(
            set(
                self.audit.ts_corporation_payments.values_list(
                    "journal__entry_id", flat=True
                )
            ),
            {1, 2},
        )
        obj = self.audit.ts_corporation_payments.get(journal__entry_id=1)
        self.assertEqual(obj.amount, 1000)
        self.assertEqual(obj.request_status, PaymentRequestStatus.APPROVED)

        obj = self.audit.ts_corporation_payments.get(journal__entry_id=2)
        self.assertEqual(obj.amount, 6000)
        self.assertEqual(obj.request_status, PaymentRequestStatus.NEEDS_APPROVAL)

    @patch(f"{MODULE_PATH}.logger")
    def test_update_tax_accounts_mark_as_missing(self, mock_logger):
        """Test should mark tax account as missing.

        Results:
            1. Mark a tax account as MISSING when the user is no longer in the corporation.
        """
        # Test Data
        tax_account = create_tax_account(
            name=self.user2_character.character.character_name,
            owner=self.audit,
            user=self.user2,
            status=AccountStatus.ACTIVE,
            deposit=0,
            last_paid=(timezone.now() - timezone.timedelta(days=30)),
        )

        # Test Action
        self.audit.update_tax_accounts(force_refresh=False)

        # Expected Results
        tax_account = CorporationPaymentAccount.objects.get(user=self.user2)
        self.assertEqual(tax_account.status, AccountStatus.MISSING)
        mock_logger.info.assert_any_call(
            "Marked Tax Account %s as MISSING",
            tax_account.name,
        )

    @patch(f"{MODULE_PATH}.logger")
    def test_update_tax_accounts_mark_as_missing_and_move_to_new_corporation(
        self, mock_logger
    ):
        """
        Test should mark tax account as missing and move to new corporation.

        Results:
            1. Move a tax account to a new corporation when the user has changed corporation.
        """
        # Test Data
        audit_2 = create_owner_from_user(self.user2)
        tax_account = create_tax_account(
            name=self.user2_character.character.character_name,
            owner=self.audit,
            user=self.user2,
            status=AccountStatus.ACTIVE,
            deposit=0,
            last_paid=(timezone.now() - timezone.timedelta(days=30)),
        )

        # Test Action
        self.audit.update_tax_accounts(force_refresh=False)

        # Expected Results
        tax_account = CorporationPaymentAccount.objects.get(user=self.user2)
        self.assertEqual(tax_account.status, AccountStatus.ACTIVE)
        self.assertEqual(tax_account.owner, audit_2)
        mock_logger.info.assert_any_call(
            "Moved Tax Account %s to Corporation %s",
            tax_account.name,
            audit_2.eve_corporation.corporation_name,
        )

    @patch(f"{MODULE_PATH}.logger")
    def test_update_tax_accounts_reset_a_returning_user(self, mock_logger):
        """
        Test should reset a tax account after a user returning to previous corporation.

        Results:
            1. Reset a tax account when the user was missing and has returned to the previous corporation.
        """
        # Test Data
        tax_account = create_tax_account(
            name=self.user_character.character.character_name,
            owner=self.audit,
            user=self.user,
            status=AccountStatus.MISSING,
            deposit=10000,
            last_paid=(timezone.now() - timezone.timedelta(days=30)),
        )

        # Test Action
        self.audit.update_tax_accounts(force_refresh=False)

        # Expected Results
        tax_account = CorporationPaymentAccount.objects.get(user=self.user)
        self.assertEqual(tax_account.deposit, 0)
        self.assertEqual(tax_account.status, AccountStatus.ACTIVE)
        self.assertEqual(tax_account.owner, self.audit)
        mock_logger.info.assert_any_call(
            "Reset Tax Account %s",
            tax_account.name,
        )

    def test_payment_deadlines(self):
        """
        Test payment deadlines processing for corporation tax accounts.
        This test should process the payment deadlines for corporation tax accounts, deducting the tax amount from the deposit.

        Results:
            1. Tax Account deposit is reduced by the tax amount on payment deadlines.
            2. New users within the free period are not charged.
        """
        # Test Data
        self.audit.tax_amount = 1000
        tax_account = create_tax_account(
            name=self.user_character.character.character_name,
            owner=self.audit,
            user=self.user,
            status=AccountStatus.ACTIVE,
            deposit=1000,
            last_paid=(timezone.now() - timezone.timedelta(days=60)),
        )
        new_user, new_user_character = create_user_from_evecharacter(
            character_id=1006,
            permissions=["taxsystem.basic_access"],
        )

        # 1 Month is free for new users
        tax_account_2 = create_tax_account(
            name=new_user_character.character.character_name,
            owner=self.audit,
            user=new_user,
            status=AccountStatus.ACTIVE,
            deposit=0,
            last_paid=None,
        )

        # Test Action
        self.audit.update_deadlines(force_refresh=False)

        # Expected Results
        tax_account = CorporationPaymentAccount.objects.get(user=self.user)
        self.assertEqual(tax_account.deposit, 0)
        tax_account_2 = CorporationPaymentAccount.objects.get(user=new_user)
        self.assertEqual(tax_account_2.deposit, 0)

    @patch(MODULE_PATH + ".esi")
    @patch(MODULE_PATH + ".EveEntity.objects.bulk_resolve_names")
    @patch(MODULE_PATH + ".logger")
    def test_update_members(self, mock_logger, mock_bulk_resolve, mock_esi):
        """
        Test update corporation members.
        This test should update or create corporation members based on ESI data.

        Results:
            1. Existing member is updated.
            2. New members are created based on ESI data.
            3. Missing members are identified.

            2 New Members created, 1 Existing updated, 1 Missing
        """
        # Test Data
        self.member = create_member(
            owner=self.audit,
            character_id=1001,
            character_name="Member 1",
            status=Members.States.ACTIVE,
        )

        self.missing_member = create_member(
            owner=self.audit,
            character_id=1004,
            character_name="Member 4",
            status=Members.States.ACTIVE,
        )

        mock_esi.client = create_esi_client_stub(
            endpoints=TEST_CORPORATION_MANAGER_ENDPOINTS
        )

        mock_bulk_resolve.return_value.to_name.side_effect = (
            "Member 1",
            "Member 2",
            "Member 3",
        )

        # Test Action
        self.audit.update_members(force_refresh=False)

        # Expected Results
        obj = self.audit.ts_members.get(character_id=1001)
        self.assertEqual(obj.character_name, "Member 1")
        obj = self.audit.ts_members.get(character_id=1002)
        self.assertEqual(obj.character_name, "Member 2")
        obj = self.audit.ts_members.get(character_id=1003)
        self.assertEqual(obj.character_name, "Member 3")

        mock_logger.info.assert_called_with(
            "%s - Old Members: %s, New Members: %s, Missing: %s",
            self.audit.eve_corporation.corporation_name,
            1,
            2,
            1,
        )

    def test_update_payments(self):
        """
        Test update corporation payments.
        This test should update or create corporation payments based on wallet journal entries.

        Results:
            1. Existing payment is updated.
            2. New payments are created based on wallet journal entries.
        """
        # Test Data
        create_tax_account(
            name=self.user_character.character.character_name,
            owner=self.audit,
            user=self.user2,
            status=AccountStatus.ACTIVE,
            deposit=0,
            last_paid=(timezone.now() - timezone.timedelta(days=30)),
        )
        self.audit2 = create_owner_from_user(self.user2)

        create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="player_donation",
            first_party=self.eve_character_first_party,
            second_party=self.eve_character_second_party,
            description="Test Description",
        )

        create_wallet_journal_entry(
            division=self.division,
            entry_id=2,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="player_donation",
            second_party=self.eve_character_second_party,
            description="Test Description",
        )
        # Test Action
        self.audit.update_payments(force_refresh=False)

        # Expected Results
        obj = self.audit.ts_corporation_payments.get(journal__entry_id=1)
        self.assertEqual(obj.amount, 1000)
        self.assertEqual(obj.request_status, PaymentRequestStatus.PENDING)

        # The second journal entry should not create a payment because it doesn't have a first_party
        with self.assertRaises(CorporationPayments.DoesNotExist):
            self.audit.ts_corporation_payments.get(journal__entry_id=2)



# Generated by Django 4.2.26 on 2025-12-23 12:14

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0022_alter_alliancepaymenthistory_new_status_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="alliancefilter",
            name="match_type",
            field=models.CharField(
                choices=[("exact", "Exact Match"), ("contains", "Contains")],
                default="exact",
                max_length=20,
            ),
        ),
        migrations.AddField(
            model_name="corporationfilter",
            name="match_type",
            field=models.CharField(
                choices=[("exact", "Exact Match"), ("contains", "Contains")],
                default="exact",
                max_length=20,
            ),
        ),
    ]



# Standard Library
import json

# Third Party
from ninja import NinjaAPI

# Django
from django.contrib.humanize.templatetags.humanize import intcomma
from django.core.handlers.wsgi import WSGIRequest
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# AA TaxSystem
from taxsystem import __title__, forms
from taxsystem.api.helpers import core
from taxsystem.api.helpers.icons import (
    get_filter_delete_button,
    get_filter_set_action_icons,
    get_filter_set_active_icon,
)
from taxsystem.api.schema import (
    DataTableSchema,
    FilterModelSchema,
    FilterSetModelSchema,
)
from taxsystem.models.corporation import (
    CorporationOwner,
)
from taxsystem.models.helpers.textchoices import (
    AdminActions,
)
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)


class FilterApiEndpoints:
    tags = ["Filter Management"]

    # pylint: disable=too-many-statements
    def __init__(self, api: NinjaAPI):
        @api.get(
            "owner/{owner_id}/filter-set/{filterset_pk}/view/filter/",
            response={200: list, 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_filters(request, owner_id: int, filterset_pk: int):
            """
            Handle an Request to retrieve Filters for a Filter Set.
            This Endpoint retrieves the filters for a filter set for a owner.
            It validates the request, checks permissions, and retrieves the according filters.

            Args:
                request (WSGIRequest): The HTTP request object.
                owner_id (int): The ID of the owner whose filter set is to be retrieved.
                filter_pk (int): The ID of the filter set whose filters are to be retrieved.
            Returns:
                list[FilterModelSchema]: A list of filter schema objects.
            """
            # pylint: disable=duplicate-code
            owner, perms = core.get_manage_owner(request, owner_id)

            if owner is None:
                return 404, {"error": _("Owner not Found.")}

            if perms is False:
                return 403, {"error": _("Permission Denied.")}

            filters = owner.filter_model.objects.filter(
                filter_set__pk=filterset_pk,
            ).select_related("filter_set", "filter_set__owner")

            response_filter_list: list[FilterModelSchema] = []
            for filter_obj in filters:
                if filter_obj.filter_type == filter_obj.__class__.FilterType.AMOUNT:
                    display = f"{intcomma(filter_obj.value)} ISK"
                else:
                    display = str(filter_obj.value)

                response_filter = FilterModelSchema(
                    filter_set=FilterSetModelSchema(
                        owner_id=filter_obj.filter_set.owner.pk,
                        name=filter_obj.filter_set.name,
                        description=filter_obj.filter_set.description,
                        enabled=filter_obj.filter_set.enabled,
                    ),
                    filter_type=filter_obj.get_filter_type_display(),
                    match_type=filter_obj.get_match_type_display(),
                    value=DataTableSchema(
                        raw=filter_obj.value,
                        display=display,
                        sort=str(filter_obj.value),
                    ),
                    actions=get_filter_delete_button(filter_obj=filter_obj),
                )
                response_filter_list.append(response_filter)

            return response_filter_list

        @api.post(
            "owner/{owner_id}/filter/{filter_pk}/manage/delete-filter/",
            response={200: dict, 403: dict, 404: dict},
            tags=self.tags,
        )
        def delete_filter(request: WSGIRequest, owner_id: int, filter_pk: int):
            """
            Handle an Request to delete a Filter.

            This Endpoint deletes a filter from an associated owner.
            It validates the request, checks permissions, and deletes the according filter.

            Args:
                request (WSGIRequest): The HTTP request object.
                owner_id (int): The ID of the owner whose filter set is to be retrieved.
                filter_pk (int): The ID of the filter to be deleted.
            Returns:
                dict: A dictionary containing the success status and message.
            """
            # pylint: disable=duplicate-code
            owner, perms = core.get_manage_owner(request, owner_id)

            # Check if owner exists
            if owner is None:
                return 404, {"error": _("Owner not Found.")}

            # Check permissions
            if perms is False:
                return 403, {"error": _("Permission Denied.")}

            # Validate the form data
            form = (
                forms.DeleteCorporationFilterForm(data=json.loads(request.body))
                if isinstance(owner, CorporationOwner)
                else forms.DeleteAllianceFilterForm(data=json.loads(request.body))
            )
            if not form.is_valid():
                msg = _("Invalid form data.")
                return 400, {"success": False, "message": msg}

            # Check if filter set exists
            filter_obj = owner.filter_model.objects.filter(
                filter_set__owner=owner, pk=filter_pk
            ).first()
            if not filter_obj:
                msg = _("Filter not found.")
                return 404, {"success": False, "message": msg}

            # Delete the filter
            filter_obj.delete()

            # Create log message
            msg = format_lazy(
                _('{filter_obj} in "{filter_set}" deleted - Reason: {reason}'),
                filter_obj=filter_obj,
                filter_set=filter_obj.filter_set.name,
                reason=form.cleaned_data["comment"],
            )
            # Log the deletion in Admin History
            owner.admin_log_model(
                user=request.user,
                owner=owner,
                action=AdminActions.DELETE,
                comment=msg,
            ).save()

            # Return success response
            return 200, {"success": True, "message": msg}

        @api.get(
            "owner/{owner_id}/view/filter-set/",
            response={200: list, 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_filter_set(request, owner_id: int):
            """
            This Endpoint retrieves the filter set for an owner.
            Args:
                request (WSGIRequest): The HTTP request object.
                owner_id (int): The ID of the owner whose filter set is to be retrieved.
            Returns:
                list[FilterSetModelSchema]: A list of filter set schema objects.
            """
            # pylint: disable=duplicate-code
            owner, perms = core.get_manage_owner(request, owner_id)

            if owner is None:
                return 404, {"error": _("Owner not Found.")}

            if perms is False:
                return 403, {"error": _("Permission Denied.")}

            filter_sets = owner.filterset_model.objects.filter(
                owner=owner,
            ).select_related("owner")

            response_filter_list: list[FilterSetModelSchema] = []
            for filter_set in filter_sets:
                response_filter = FilterSetModelSchema(
                    owner_id=filter_set.owner.pk,
                    name=filter_set.name,
                    description=filter_set.description,
                    enabled=filter_set.enabled,
                    status=DataTableSchema(
                        raw=filter_set.enabled,
                        display=get_filter_set_active_icon(filter_set=filter_set),
                        sort=str(int(filter_set.enabled)),
                    ),
                    actions=get_filter_set_action_icons(
                        request=request, filter_set=filter_set
                    ),
                )
                response_filter_list.append(response_filter)

            return response_filter_list

        @api.post(
            "owner/{owner_id}/filter-set/{filterset_pk}/manage/delete-filter/",
            response={200: dict, 403: dict, 404: dict},
            tags=self.tags,
        )
        def delete_filter_set(request: WSGIRequest, owner_id: int, filterset_pk: int):
            """
            Handle an Request to delete a Filter Set.

            This Endpoint deletes a filter set for a owner.
            It validates the request, checks permissions, and deletes the according filter set.

            Args:
                request (WSGIRequest): The HTTP request object.
                owner_id (int): The ID of the owner whose filter set is to be retrieved.
                filter_pk (int): The ID of the filter to be deleted.
            Returns:
                dict: A dictionary containing the success status and message.
            """
            # pylint: disable=duplicate-code
            owner, perms = core.get_manage_owner(request, owner_id)

            # Check if owner exists
            if owner is None:
                return 404, {"error": _("Owner not Found.")}

            # Check permissions
            if perms is False:
                return 403, {"error": _("Permission Denied.")}

            # Validate the form data
            form = (
                forms.DeleteCorporationFilterSetForm(data=json.loads(request.body))
                if isinstance(owner, CorporationOwner)
                else forms.DeleteAllianceFilterSetForm(data=json.loads(request.body))
            )
            if not form.is_valid():
                msg = _("Invalid form data.")
                return 400, {"success": False, "message": msg}

            # Check if filter set exists
            filter_set = owner.filterset_model.objects.filter(
                owner=owner, pk=filterset_pk
            ).first()
            if not filter_set:
                msg = _("Filter Set not found.")
                return 404, {"success": False, "message": msg}

            # Delete the filter set
            filter_set.delete()

            # Create log message
            msg = format_lazy(
                _("{filter_set} deleted - Reason: {reason}"),
                filter_set=filter_set,
                reason=form.cleaned_data["comment"],
            )

            # Log the deletion in Admin History
            owner.admin_log_model(
                user=request.user,
                owner=owner,
                action=AdminActions.DELETE,
                comment=msg,
            ).save()

            # Return success response
            return 200, {"success": True, "message": msg}

        @api.post(
            "owner/{owner_id}/filter-set/{filterset_pk}/manage/switch-filter/",
            response={200: dict, 403: dict, 404: dict},
            tags=self.tags,
        )
        def switch_filter_set(request: WSGIRequest, owner_id: int, filterset_pk: int):
            """
            Handle an Request to Switch a Filter Set.

            This Endpoint handle an Request to switching a filter set from an associated owner depending on its current state.
            It validates the request, checks permissions, and toggles the enabled state of the according filter set.

            Args:
                request (WSGIRequest): The HTTP request object.
                owner_id (int): The ID of the owner whose filter set is to be retrieved.
                filter_pk (int): The ID of the filter set to be switched.
            Returns:
                dict: A dictionary containing the success status and message.
            """
            # pylint: disable=duplicate-code
            owner, perms = core.get_manage_owner(request, owner_id)

            # Check if owner exists
            if owner is None:
                return 404, {"error": _("Owner not Found.")}

            # Check permissions
            if perms is False:
                return 403, {"error": _("Permission Denied.")}

            # Check if filter set exists
            filter_set = owner.filterset_model.objects.filter(
                owner=owner, pk=filterset_pk
            ).first()
            if not filter_set:
                msg = _("Filter Set not found.")
                return 404, {"success": False, "message": msg}

            # Toggle the filter set enabled state
            filter_set.enabled = not filter_set.enabled
            filter_set.save()

            # Create log message
            msg = format_lazy(
                _("{filter_set} switched to {enabled}"),
                filter_set=filter_set,
                enabled=filter_set.enabled,
            )

            # Return success response
            return 200, {"success": True, "message": msg}



# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# AA TaxSystem
from taxsystem import __title__
from taxsystem.decorators import (
    log_timing,
)
from taxsystem.providers import AppLogger
from taxsystem.tests import NoSocketsTestCase

DECORATOR_PATH = "taxsystem.decorators."


class TestDecorators(NoSocketsTestCase):
    def test_log_timing(self):
        """
        Test should log execution time of decorated function.
        """
        # given
        logger = AppLogger(my_logger=get_extension_logger(__name__), prefix=__title__)

        @log_timing(logger)
        def trigger_log_timing():
            return "Log Timing"

        # when
        result = trigger_log_timing()
        # then
        self.assertEqual(result, "Log Timing")



"""App Tasks"""

# Standard Library
import inspect
from collections.abc import Callable

# Third Party
from celery import Task, chain, shared_task

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger
from allianceauth.services.tasks import QueueOnce

# AA TaxSystem
from taxsystem import __title__, app_settings
from taxsystem.models.alliance import AllianceOwner
from taxsystem.models.corporation import CorporationOwner
from taxsystem.models.helpers.textchoices import (
    AllianceUpdateSection,
    CorporationUpdateSection,
)
from taxsystem.providers import AppLogger, retry_task_on_esi_error

logger = AppLogger(get_extension_logger(__name__), __title__)

MAX_RETRIES_DEFAULT = 3

# Default params for all tasks.
TASK_DEFAULTS = {
    "time_limit": app_settings.TAXSYSTEM_TASKS_TIME_LIMIT,
    "max_retries": MAX_RETRIES_DEFAULT,
}

# Default params for tasks that need bind=True and run once only.
TASK_DEFAULTS_BIND_ONCE = {**TASK_DEFAULTS, **{"bind": True, "base": QueueOnce}}

# Default params for tasks that need run once only.
TASK_DEFAULTS_ONCE = {**TASK_DEFAULTS, **{"base": QueueOnce}}

TASK_DEFAULTS_BIND_ONCE_OWNER = {
    **TASK_DEFAULTS_BIND_ONCE,
    **{"once": {"keys": ["owner_eve_id"], "graceful": True}},
}


@shared_task(**TASK_DEFAULTS_ONCE)
def update_all_taxsytem(runs: int = 0, force_refresh: bool = False):
    """Update all taxsystem data"""
    corporations: list[CorporationOwner] = CorporationOwner.objects.select_related(
        "eve_corporation"
    ).filter(active=1)

    alliances: list[AllianceOwner] = AllianceOwner.objects.select_related(
        "eve_alliance"
    ).filter(active=1)
    # Queue tasks for all corporations
    for corporation in corporations:
        update_corporation.apply_async(
            args=[corporation.eve_id], kwargs={"force_refresh": force_refresh}
        )
        runs = runs + 1
    # Queue tasks for all alliances
    for alliance in alliances:
        update_alliance.apply_async(
            args=[alliance.eve_id], kwargs={"force_refresh": force_refresh}
        )
        runs = runs + 1
    logger.info("Queued %s Owner Tasks", runs)


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_corporation(
    self: Task,  # pylint: disable=unused-argument
    owner_eve_id: int,
    force_refresh: bool = False,
) -> bool:
    """Update a corporation"""
    owner: CorporationOwner = CorporationOwner.objects.prefetch_related(
        "ts_corporation_update_status"
    ).get(eve_corporation__corporation_id=owner_eve_id)

    que = []
    priority = 7

    logger.debug(
        "Processing Audit Updates for %s",
        format(owner.name),
    )

    if force_refresh:
        # Reset Token Error if we are forcing a refresh
        owner.update_manager.reset_has_token_error()

    needs_update = owner.update_manager.calc_update_needed()

    if not needs_update and not force_refresh:
        logger.info("No updates needed for %s", owner.name)
        return False

    sections = CorporationUpdateSection.get_sections()

    for section in sections:
        # Skip sections that are not in the needs_update list
        if not force_refresh and not needs_update.for_section(section):
            logger.debug(
                "No updates needed for %s (%s)",
                owner.name,
                section,
            )
            continue

        task_name = f"update_corp_{section}"
        task = globals().get(task_name)
        que.append(
            task.si(owner.eve_id, force_refresh=force_refresh).set(priority=priority)
        )

    chain(que).apply_async()
    logger.debug(
        "Queued %s Audit Updates for %s",
        len(que),
        owner.name,
    )
    return True


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_corp_division_names(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_corp_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=CorporationUpdateSection.DIVISION_NAMES,
        force_refresh=force_refresh,
    )


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_corp_divisions(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_corp_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=CorporationUpdateSection.DIVISIONS,
        force_refresh=force_refresh,
    )


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_corp_wallet(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_corp_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=CorporationUpdateSection.WALLET,
        force_refresh=force_refresh,
    )


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_corp_members(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_corp_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=CorporationUpdateSection.MEMBERS,
        force_refresh=force_refresh,
    )


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_corp_payments(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_corp_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=CorporationUpdateSection.PAYMENTS,
        force_refresh=force_refresh,
    )


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_corp_tax_accounts(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_corp_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=CorporationUpdateSection.TAX_ACCOUNTS,
        force_refresh=force_refresh,
    )


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_corp_deadlines(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_corp_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=CorporationUpdateSection.DEADLINES,
        force_refresh=force_refresh,
    )


def _update_corp_section(
    task: Task, owner_eve_id: int, section: str, force_refresh: bool
):
    """Update a specific section of the corporation."""
    section = CorporationUpdateSection(section)
    owner = CorporationOwner.objects.get(eve_corporation__corporation_id=owner_eve_id)
    logger.debug("Updating %s for %s", section.label, owner.name)

    owner.update_manager.reset_update_status(section)

    method: Callable = getattr(owner, section.method_name)
    method_signature = inspect.signature(method)

    if "force_refresh" in method_signature.parameters:
        kwargs = {"force_refresh": force_refresh}
    else:
        kwargs = {}

    # Perform the update with the retry context manager
    with retry_task_on_esi_error(task):
        result = owner.update_manager.perform_update_status(section, method, **kwargs)
    owner.update_manager.update_section_log(section, result)


# Alliance Tasks


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_alliance(
    self: Task,  # pylint: disable=unused-argument
    owner_eve_id: int,
    force_refresh: bool = False,
) -> bool:
    """Update an alliance"""
    owner: AllianceOwner = AllianceOwner.objects.prefetch_related(
        "ts_alliance_update_status"
    ).get(eve_alliance__alliance_id=owner_eve_id)

    que = []
    priority = 7

    logger.debug(
        "Processing Audit Updates for %s",
        format(owner.name),
    )

    if force_refresh:
        # Reset Token Error if we are forcing a refresh
        owner.update_manager.reset_has_token_error()

    needs_update = owner.update_manager.calc_update_needed()

    if not needs_update and not force_refresh:
        logger.info("No updates needed for %s", owner.name)
        return False

    sections = AllianceUpdateSection.get_sections()

    for section in sections:
        # Skip sections that are not in the needs_update list
        if not force_refresh and not needs_update.for_section(section):
            logger.debug(
                "No updates needed for %s (%s)",
                owner.name,
                section,
            )
            continue

        task_name = f"update_ally_{section}"
        task = globals().get(task_name)
        que.append(
            task.si(owner.eve_id, force_refresh=force_refresh).set(priority=priority)
        )

    chain(que).apply_async()
    logger.debug(
        "Queued %s Audit Updates for %s",
        len(que),
        owner.name,
    )
    return True


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_ally_payments(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_ally_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=AllianceUpdateSection.PAYMENTS,
        force_refresh=force_refresh,
    )


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_ally_tax_accounts(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_ally_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=AllianceUpdateSection.TAX_ACCOUNTS,
        force_refresh=force_refresh,
    )


@shared_task(**TASK_DEFAULTS_BIND_ONCE_OWNER)
def update_ally_deadlines(self: Task, owner_eve_id: int, force_refresh: bool):
    return _update_ally_section(
        task=self,
        owner_eve_id=owner_eve_id,
        section=AllianceUpdateSection.DEADLINES,
        force_refresh=force_refresh,
    )


def _update_ally_section(
    task: Task, owner_eve_id: int, section: str, force_refresh: bool
):
    """Update a specific section of the alliance."""
    section = AllianceUpdateSection(section)
    alliance = AllianceOwner.objects.get(eve_alliance__alliance_id=owner_eve_id)
    logger.debug("Updating %s for %s", section.label, alliance.name)
    alliance.update_manager.reset_update_status(section)

    method: Callable = getattr(alliance, section.method_name)
    method_signature = inspect.signature(method)

    if "force_refresh" in method_signature.parameters:
        kwargs = {"force_refresh": force_refresh}
    else:
        kwargs = {}

    # Perform the update with the retry context manager
    with retry_task_on_esi_error(task):
        result = alliance.update_manager.perform_update_status(
            section, method, **kwargs
        )
    alliance.update_manager.update_section_log(section, result)



"""PvE Views"""

# Django
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.db import IntegrityError, transaction
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.authentication.models import UserProfile
from allianceauth.eveonline.models import (
    EveAllianceInfo,
    EveCharacter,
    EveCorporationInfo,
)
from allianceauth.services.hooks import get_extension_logger
from esi.decorators import token_required

# AA TaxSystem
from taxsystem import __title__, forms, tasks
from taxsystem.api.helpers.core import (
    get_character_permissions,
    get_manage_owner,
    get_owner,
)
from taxsystem.helpers import lazy
from taxsystem.models.alliance import (
    AllianceOwner,
)
from taxsystem.models.corporation import (
    CorporationOwner,
    Members,
)
from taxsystem.models.helpers.textchoices import AccountStatus, AdminActions
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)


@login_required
@permission_required("taxsystem.basic_access")
def admin(request: WSGIRequest):
    corporation_id = request.user.profile.main_character.corporation_id
    if not request.user.is_superuser:
        messages.error(request, _("You do not have permission to access this page."))
        return redirect("taxsystem:index")

    def _handle_taxsystem_updates(force_refresh):
        messages.info(request, _("Queued Update All Taxsystem"))
        tasks.update_all_taxsytem.apply_async(
            kwargs={"force_refresh": force_refresh}, priority=7
        )

    def _handle_corporation_updates(force_refresh):
        corporation_id_input = request.POST.get("corporation_id")
        if corporation_id_input:
            try:
                corp_id = int(corporation_id_input)
                corporation = CorporationOwner.objects.get(
                    eve_corporation__corporation_id=corp_id
                )
                messages.info(
                    request,
                    _("Queued Update for Corporation: %s") % corporation.name,
                )
                tasks.update_corporation.apply_async(
                    args=[corporation.eve_id],
                    kwargs={"force_refresh": force_refresh},
                    priority=7,
                )
            except (ValueError, CorporationOwner.DoesNotExist):
                messages.error(
                    request,
                    _("Corporation with ID %s not found") % corporation_id_input,
                )
        else:
            messages.info(request, _("Queued Update All Taxsystem Corporations"))
            corporations = CorporationOwner.objects.filter(active=True)
            for corporation in corporations:
                tasks.update_corporation.apply_async(
                    args=[corporation.eve_id],
                    kwargs={"force_refresh": force_refresh},
                    priority=7,
                )

    def _handle_alliance_updates(force_refresh):
        alliance_id_input = request.POST.get("alliance_id")
        if alliance_id_input:
            try:
                ally_id = int(alliance_id_input)
                alliance = AllianceOwner.objects.get(eve_alliance__alliance_id=ally_id)
                messages.info(
                    request,
                    _("Queued Update for Alliance: %s") % alliance.name,
                )
                tasks.update_alliance.apply_async(
                    args=[alliance.eve_alliance.alliance_id],
                    kwargs={"force_refresh": force_refresh},
                    priority=7,
                )
            except (ValueError, AllianceOwner.DoesNotExist):
                messages.error(
                    request,
                    _("Alliance with ID %s not found") % alliance_id_input,
                )
        else:
            messages.info(request, _("Queued Update All Taxsystem Alliances"))
            alliances = AllianceOwner.objects.filter(active=True)
            for alliance in alliances:
                tasks.update_alliance.apply_async(
                    args=[alliance.eve_alliance.alliance_id],
                    kwargs={"force_refresh": force_refresh},
                    priority=7,
                )

    if request.method == "POST":
        force_refresh = bool(request.POST.get("force_refresh", False))
        if request.POST.get("run_taxsystem_updates"):
            _handle_taxsystem_updates(force_refresh)
        if request.POST.get("run_taxsystem_corporation_updates"):
            _handle_corporation_updates(force_refresh)
        if request.POST.get("run_taxsystem_alliance_updates"):
            _handle_alliance_updates(force_refresh)

    context = {
        "corporation_id": corporation_id,
        "title": _("Tax System Superuser Administration"),
    }
    return render(request, "taxsystem/admin.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def index(request: WSGIRequest):  # pylint: disable=unused-argument
    """
    Main TaxSystem View
    This view displays an overview of all available tax system owners depending on user's permissions.
    Args:
        request (WSGIRequest): The HTTP request object containing user information from Alliance Auth
    Returns:
        HttpResponse: Rendered owner overview template with combined owner list
    """
    owner_list = []

    # Get all Corporations the user can see with their permissions
    corporations = (
        CorporationOwner.objects.visible_to(request.user)
        .select_related("eve_corporation")
        .order_by("eve_corporation__corporation_name")
    )
    for corporation in corporations:
        owner_list.append(
            {
                "type": "corporation",
                "type_display": _("Corporation"),
                "id": corporation.eve_corporation.corporation_id,
                "name": corporation.eve_corporation.corporation_name,
                "portrait": lazy.get_corporation_logo_url(
                    corporation.eve_corporation.corporation_id,
                    size=64,
                    corporation_name=corporation.eve_corporation.corporation_name,
                    as_html=True,
                ),
                "active": corporation.active,
                "open_invoices": corporation.payment_model.objects.get_owner_open_invoices(
                    user=request.user, owner=corporation
                ),
                "actions": "",
            }
        )

    alliances = (
        AllianceOwner.objects.visible_to(request.user)
        .select_related("eve_alliance")
        .order_by("eve_alliance__alliance_name")
    )
    for alliance in alliances:
        owner_list.append(
            {
                "type": "alliance",
                "type_display": _("Alliance"),
                "id": alliance.eve_alliance.alliance_id,
                "name": alliance.eve_alliance.alliance_name,
                "portrait": lazy.get_alliance_logo_url(
                    alliance.eve_alliance.alliance_id,
                    size=64,
                    alliance_name=alliance.eve_alliance.alliance_name,
                    as_html=True,
                ),
                "active": alliance.active,
                "open_invoices": alliance.payment_model.objects.get_owner_open_invoices(
                    user=request.user, owner=alliance
                ),
                "actions": "",
            }
        )

    context = {
        "owners": owner_list,
        "title": _("TaxSystem Overview"),
        "total_count": len(owner_list),
    }

    return render(request, "taxsystem/taxsystem.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def payments(request: WSGIRequest, owner_id: int):
    """
    Render Payments View
    This view displays the payments made to a specific owner depending on user's permissions.
    Args:
        request (WSGIRequest): The HTTP request object containing user information from Alliance Auth
        owner_id (int): The ID of the owner for which to display payments
    Returns:
        HttpResponse: Rendered payments template with payment details
    """
    owner, perms = get_owner(request, owner_id)
    if owner is None:
        messages.error(request, _("Owner not Found."))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied."))
        return redirect("taxsystem:index")

    context = {
        "owner": owner,
        "title": _("Payments"),
        "forms": {
            "accept_payment_request": (
                forms.AcceptCorporationPaymentForm()
                if isinstance(owner, CorporationOwner)
                else forms.AcceptAlliancePaymentForm()
            ),
            "accept_reject_payment_request": (
                forms.RejectCorporationPaymentForm()
                if isinstance(owner, CorporationOwner)
                else forms.RejectAlliancePaymentForm()
            ),
            "accept_undo_payment_request": (
                forms.UndoCorporationPaymentForm()
                if isinstance(owner, CorporationOwner)
                else forms.UndoAlliancePaymentForm()
            ),
            "accept_delete_payment_request": (
                forms.DeleteCorporationPaymentForm()
                if isinstance(owner, CorporationOwner)
                else forms.DeleteAlliancePaymentForm()
            ),
        },
    }

    return render(request, "taxsystem/view-payments.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def my_payments(request: WSGIRequest, owner_id: int):
    """
    Render My Payments View
    This view displays the payments made by the logged-in user for a specific owner depending on user's permissions.
    Args:
        request (WSGIRequest): The HTTP request object containing user information from Alliance Auth
        owner_id (int): The ID of the owner for which to display payments
    Returns:
        HttpResponse: Rendered my payments template with payment details
    """
    owner, perms = get_owner(request, owner_id)

    if owner is None:
        messages.error(request, _("Owner not Found."))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied."))
        return redirect("taxsystem:index")

    context = {
        "owner": owner,
        "title": _("My Payments"),
    }
    return render(request, "taxsystem/view-my-payments.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def faq(request: WSGIRequest, owner_id: int):
    """
    Render FAQ View
    This view displays the FAQ for a specific owner depending on user's permissions.
    Args:
        request (WSGIRequest): The HTTP request object containing user information from Alliance Auth
        owner_id (int): The ID of the owner to retrieve
    Returns:
        HttpResponse: Rendered FAQ template with owner details
    """
    owner, perms = get_owner(request, owner_id)
    if owner is None:
        messages.error(request, _("Owner not Found."))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied."))
        return redirect("taxsystem:index")

    context = {
        "owner": owner,
        "title": _("FAQ"),
    }

    return render(request, "taxsystem/view-faq.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def account(request: WSGIRequest, owner_id: int, character_id: int = None):
    """
    Render Account View
    This view displays the account details for a specific character and owner depending on user's permissions.
    Args:
        request (WSGIRequest): The HTTP request object containing user information from Alliance Auth
        owner_id (int): The ID of the owner to retrieve
        character_id (int): The ID of the character whose account details are to be displayed
    Returns:
        HttpResponse: Rendered account template with account details
    """
    if character_id is None:
        character_id = request.user.profile.main_character.character_id

    user_profile = UserProfile.objects.filter(
        main_character__character_id=character_id
    ).first()

    if not user_profile:
        messages.error(request, _("No User found."))
        return redirect("taxsystem:index")

    owner, perms = get_manage_owner(request, owner_id)
    # Give access if character permission exists
    perms = perms or get_character_permissions(request, character_id)

    if owner is None:
        messages.error(request, _("Owner not Found."))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied."))
        return redirect("taxsystem:index")

    tax_account = owner.account_model.objects.filter(
        user__profile=user_profile,
        owner=owner,
    ).first()

    if not tax_account:
        messages.error(request, _("No Tax Account found."))
        return redirect("taxsystem:index")

    # Get member info
    try:
        member = Members.objects.get(character_id=character_id)
    except Members.DoesNotExist:
        member = None

    context = {
        "title": _("Account"),
        "owner": owner,
        "account": {
            "name": tax_account.name,
            "owner": owner,
            "corporation": owner,
            "character_id": character_id,
            "status": AccountStatus(tax_account.status).html(text=True),
            "deposit": (
                tax_account.deposit
                if tax_account.status != AccountStatus.MISSING
                else "N/A"
            ),
            "has_paid": (
                tax_account.has_paid_icon(badge=True, text=True)
                if tax_account.status != AccountStatus.MISSING
                else "N/A"
            ),
            "last_paid": (
                tax_account.last_paid
                if tax_account.status != AccountStatus.MISSING
                else "N/A"
            ),
            "next_due": (
                tax_account.next_due
                if tax_account.status != AccountStatus.MISSING
                else "N/A"
            ),
            "joined": member.joined if member else "N/A",
            "last_login": member.logon if member else "N/A",
        },
    }

    return render(request, "taxsystem/view-account.html", context=context)


@login_required
def manage_owner(request: WSGIRequest, owner_id: int = None):
    """Manage View (Backwards-compatible wrapper)"""
    owner, perms = get_manage_owner(request, owner_id)

    if owner is None:
        messages.error(request, _("Owner not Found."))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied."))
        return redirect("taxsystem:index")

    context = {
        "owner": owner,
        "title": _("Manage Tax System"),
        "manage_filter_url": reverse(
            "taxsystem:manage_filter", kwargs={"owner_id": owner_id}
        ),
        "forms": {
            "accept_payment_request": (
                forms.AcceptCorporationPaymentForm()
                if isinstance(owner, CorporationOwner)
                else forms.AcceptAlliancePaymentForm()
            ),
            "accept_reject_payment_request": (
                forms.RejectCorporationPaymentForm()
                if isinstance(owner, CorporationOwner)
                else forms.RejectAlliancePaymentForm()
            ),
            "add_payment_request": forms.PaymentAddForm(),
            "accept_delete_payment_request": (
                forms.DeleteCorporationPaymentForm()
                if isinstance(owner, CorporationOwner)
                else forms.DeleteAlliancePaymentForm()
            ),
            "accept_undo_payment_request": (
                forms.UndoCorporationPaymentForm()
                if isinstance(owner, CorporationOwner)
                else forms.UndoAlliancePaymentForm()
            ),
            "accept_delete_member_request": forms.DeleteMemberForm(),
        },
    }
    return render(request, "taxsystem/view-manage.html", context=context)


@login_required
def manage_filter(request: WSGIRequest, owner_id: int):
    """Manage View"""
    owner, perms = get_manage_owner(request, owner_id)

    if perms is False:
        messages.error(request, _("You do not have permission to manage this owner."))
        return redirect("taxsystem:index")

    # Get existing filters
    queryset = owner.filterset_model.objects.filter(owner=owner)

    context = {
        "owner": owner,
        "title": _("Manage Filters"),
        "forms": {
            "filter": forms.AddJournalFilterForm(queryset=queryset),
            "filter_set": forms.CreateFilterSetForm(),
            "delete_filter_set": (
                forms.DeleteCorporationFilterSetForm()
                if isinstance(owner, CorporationOwner)
                else forms.DeleteAllianceFilterSetForm()
            ),
            "delete_filter": (
                forms.DeleteCorporationFilterForm()
                if isinstance(owner, CorporationOwner)
                else forms.DeleteAllianceFilterForm()
            ),
        },
    }

    with transaction.atomic():
        form_add = forms.AddJournalFilterForm(
            data=request.POST,
            queryset=queryset,
        )
        form_set = forms.CreateFilterSetForm(data=request.POST)

        if form_add.is_valid():
            queryset = form_add.cleaned_data["filter_set"]
            filter_type = form_add.cleaned_data["filter_type"]
            match_type = form_add.cleaned_data["match_type"]
            value = form_add.cleaned_data["value"]
            try:
                owner.filter_model.objects.create(
                    filter_set=queryset,
                    filter_type=filter_type,
                    match_type=match_type,
                    value=value,
                )
            except IntegrityError:
                messages.error(
                    request,
                    format_lazy(
                        _("A Filter with {filter} already exists."),
                        filter=f'{filter_type} "{value}"',
                    ),
                )
            except Exception as e:  # pylint: disable=broad-except
                messages.error(
                    request, _("Something went wrong, please try again later.")
                )
                logger.exception("Error creating journal filter: %s", e)
                return redirect("taxsystem:manage_filter", owner_id=owner_id)

        if form_set.is_valid():
            name = form_set.cleaned_data["name"]
            description = form_set.cleaned_data["description"]
            try:
                owner.filterset_model.objects.create(
                    owner=owner,
                    name=name,
                    description=description,
                )
            except IntegrityError:
                messages.error(
                    request, _("A filter set with this name already exists.")
                )
                return redirect("taxsystem:manage_filter", owner_id=owner_id)
            except Exception as e:  # pylint: disable=broad-except
                messages.error(
                    request, _("Something went wrong, please try again later.")
                )
                logger.exception("Error creating journal filter set: %s", e)
                return redirect("taxsystem:manage_filter", owner_id=owner_id)

    return render(request, "taxsystem/view-filter.html", context=context)


@login_required
@permission_required("taxsystem.create_access")
@token_required(scopes=CorporationOwner.get_esi_scopes())
def add_corp(request: WSGIRequest, token):
    char = get_object_or_404(EveCharacter, character_id=token.character_id)
    corp, __ = EveCorporationInfo.objects.get_or_create(
        corporation_id=char.corporation_id,
        defaults={
            "member_count": 0,
            "corporation_ticker": char.corporation_ticker,
            "corporation_name": char.corporation_name,
        },
    )

    owner, created = CorporationOwner.objects.update_or_create(
        eve_corporation=corp,
        defaults={
            "name": char.corporation_name,
            "active": True,
        },
    )

    if created:
        owner.admin_log_model(
            user=request.user,
            owner=owner,
            action=AdminActions.ADD,
            comment=_("Added to Tax System"),
        ).save()

    tasks.update_corporation.apply_async(
        args=[owner.eve_id], kwargs={"force_refresh": True}, priority=6
    )
    msg = _("{corporation_name} successfully added/updated to Tax System").format(
        corporation_name=char.corporation_name,
    )
    messages.info(request, msg)
    return redirect("taxsystem:index")


@login_required
@permission_required("taxsystem.create_access")
@token_required(scopes=CorporationOwner.get_esi_scopes())
def add_alliance(request: WSGIRequest, token):
    # TODO Implement Alliance System
    char = get_object_or_404(EveCharacter, character_id=token.character_id)
    tax_corp = get_object_or_404(
        CorporationOwner, eve_corporation__corporation_id=char.corporation_id
    )

    ally, __ = EveAllianceInfo.objects.get_or_create(
        alliance_id=char.alliance_id,
        defaults={
            "member_count": 0,
            "alliance_ticker": char.alliance_ticker,
            "alliance_name": char.alliance_name,
        },
    )

    owner_alliance, created = AllianceOwner.objects.update_or_create(
        eve_alliance=ally,
        defaults={
            "corporation": tax_corp,
            "name": char.alliance_name,
            "active": True,
        },
    )

    if created:
        owner_alliance.admin_log_model(
            user=request.user,
            owner=owner_alliance,
            action=AdminActions.ADD,
            comment=_("Added Alliance to Tax System with Corporation {corp}").format(
                corp=tax_corp.name
            ),
        ).save()

    tasks.update_alliance.apply_async(
        args=[owner_alliance.eve_id], kwargs={"force_refresh": True}, priority=6
    )
    msg = _("{alliance_name} successfully added/updated to Tax System").format(
        alliance_name=char.alliance_name,
    )
    messages.info(request, msg)
    return redirect("taxsystem:index")



# Generated by Django 4.2.26 on 2025-11-19 21:26

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "taxsystem",
            "0014_rename_allianceownerupdatestatus_allianceupdatestatus_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                    ("manage_own_alliance", "Can manage own Alliance"),
                    ("manage_alliances", "Can manage all Alliances"),
                ),
            },
        ),
    ]
